package nakov1_4;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * MailServer main class.
 * When started, launches SMTP and POP3 listeners.
 * Contains user-dependent objects used for mail input/output operations.
 * @author George Denchev
 * @version 1.21
 *
 */
public class MailServer extends Thread {
	public UserLock[] locks;
	private BufferedReader in;
	private Vector users= new Vector();
	private SMTPListener smtpListener;
	private POP3Listener pop3Listener;

  /**
	 *
	 */
  public MailServer() {
		try {
			in= new BufferedReader(new FileReader("users.txt"));
	  }
		catch (FileNotFoundException fnfe) {
			System.out.println("MailServer: Users Data Not Found!");
			System.exit(-1);
		}
		try {
			String user= in.readLine();
			while (user!= null) {
				users.add(user);
				user= in.readLine();
			}
		}
		catch (IOException ioe) {
			System.out.println("MailServer: Error Reading User Data!");
			System.exit(-1);
		}
		locks= new UserLock[users.size()];
		for (int i= 0; i< users.size(); i++)
			locks[i]= new UserLock();

		smtpListener= new SMTPListener(this);
		smtpListener.setDaemon(true);
		smtpListener.start();
		pop3Listener= new POP3Listener(this);
		pop3Listener.setDaemon(true);
		pop3Listener.start();
  }

	/**
	 *
	 */
  public void run() {
		while (true) {
			try {
				sleep(10000);
			}
			catch (InterruptedException ie) {
				System.out.println("MailServer: Interrupted!");
				System.exit(-1);
			}
		}
	}

	/**
   * @param userName username to check for
   */
  public int userExists(String userName) {
		for (int i= 0; i< users.size(); i++)
			if (((String)users.elementAt(i)).equals(userName))
				return i;
		return -1;
	}

	/**
   * @param args main method command-line arguments
   */
  public static void main(String args[]) {
		MailServer mail= new MailServer();
		mail.start();
		System.out.println("MailServer: Started");
	}
}