package nakov1_4;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Helper class handling POP3 commands/operations.
 * Launched separately for each user.
 * @author George Denchev
 * @version 1.21
 *
 */
public class POP3Thread extends Thread {
	private BufferedReader in= null;
	private PrintWriter out= null;
	private Socket client= null;
	private MailServer ref= null;
	private int currentState= 0;
	private String command= null;
	private String token= null;
	private String userName= null;
	private int userIndex= -1;
	private int msgIndex= -1;
	private File userDir= null;

	/**
   * @param aClient client connection socket
   * @param ref reference to the main MailServer thread
   */
  public POP3Thread(Socket aClient, MailServer ref) {
		this.client= aClient;
		this.ref= ref;
		try {
			in= new BufferedReader(new InputStreamReader(aClient.getInputStream()));
			out= new PrintWriter(new OutputStreamWriter(aClient.getOutputStream()), true);
			out.println("+OK POP3 server ready");
		}
		catch (IOException ioe) {
			System.out.println("SMTPThread: IO Initialization Error!");
		}
	}

	/**
	 *
	 */
  public void run() {
		try {
		  while (true) {
				command= in.readLine();

				//Check If Client Quitting...
				if (command.toUpperCase().equals("QUIT")) {
					out.println("+OK Bye");
					in.close();
					out.close();
					client.close();
					break;
				}

				switch(currentState) {
					case 0: {  //Initial state
						StringTokenizer tok= new StringTokenizer(command, " ");
						token= tok.nextToken();
						if (token.toUpperCase().equals("USER")) {
						  if (tok.countTokens()== 1) {
								userName= tok.nextToken();
								out.println("+User OK, please send your password");
								currentState= 1;
							}
							else
								out.println("-ERR Invalid command");
						}
						else if ((token.toUpperCase().equals("PASS")) || (token.toUpperCase().equals("LIST")) || (token.toUpperCase().equals("RETR")))
							out.println("-ERR Invalid state");
						else
							out.println("-ERR Invalid command");
					} break;
					case 1: {  //USER state
						StringTokenizer tok= new StringTokenizer(command, " ");
						token= tok.nextToken();
						if (token.toUpperCase().equals("PASS")) {
						  if (tok.countTokens()== 1) {
								String tempPass= tok.nextToken();
								userIndex= ref.userExists(userName);
								if ((userIndex!= -1) && (userName.equals(tempPass))) {
									out.println("+OK");
								  currentState= 2;
								}
								else {
									out.println("-ERR Invalid username/password");
									currentState= 0;
								}
							}
							else
								out.println("-ERR Invalid command");
						}
						else if ((token.toUpperCase().equals("USER")) || (token.toUpperCase().equals("LIST")) || (token.toUpperCase().equals("RETR")))
							out.println("-ERR Invalid state");
						else
							out.println("-ERR Invalid command");
					} break;
					case 2: {  //PASS state
						StringTokenizer tok= new StringTokenizer(command, " ");
						token= tok.nextToken();
						if (token.toUpperCase().equals("LIST")) {
							String[] messages= new File("./users/"+ userName).list();
							out.println("+OK "+ messages.length+ " messages");
							for (int i= 0; i< messages.length; i++)
								out.println((i+1)+ " "+ new File("./users/"+ userName+ "/"+ messages[i]).length());
							out.println(".");
						}
						else if (token.toUpperCase().equals("RETR")) {
							if (tok.countTokens()== 1) {
								try {
									msgIndex= Integer.parseInt(tok.nextToken());
									userDir= new File("./users/"+ userName);
									if ((msgIndex< 1) || (msgIndex> userDir.list().length)) {
										out.println("-ERR No such message");
									}
									else {
		  							//Read Message (User-Synchronized)
										synchronized(ref.locks[userIndex]) {
											out.println("+OK Message "+ msgIndex);
											BufferedReader fileReader= new BufferedReader(new FileReader("./users/"+ userName+ "/"+ userDir.list()[msgIndex- 1]));
											String tempLine= fileReader.readLine();
											while (tempLine!= null) {
												out.println(tempLine);
												tempLine= fileReader.readLine();
											}
											out.println(".");
										}
									}
								}
								catch (NumberFormatException nfe) {
									out.println("-ERR Invalid command");
								}
							}
							else
								out.println("-ERR Invalid command");
						}
						else if ((token.toUpperCase().equals("USER")) || (token.toUpperCase().equals("PASS")))
							out.println("-ERR Invalid state");
						else
							out.println("-ERR Invalid command");
					} break;
				}
			}
		}
		catch (IOException ioe) {
			System.out.println("POP3Thread: IO Error");
		}
	}
}