package nakov1_4;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * @author George Denchev
 * @version 1.21
 *
 */
public class SMTPThread extends Thread {
	private BufferedReader in= null;
	private PrintWriter out= null;
	private Socket client= null;
	private MailServer ref= null;
	private int currentState= 0;
	private String command= null;
	private String token= null;
	private String userName= null;
	private int userIndex= -1;
	private String msgLines= "";

	/**
   * Helper class handling SMTP commands/operations.
   * Launched separately for each user.
   * @param aClient client connection socket
   * @param ref reference to the main MailServer thread
   */
  public SMTPThread(Socket aClient, MailServer ref) {
		this.client= aClient;
		this.ref= ref;
		try {
			in= new BufferedReader(new InputStreamReader(aClient.getInputStream()));
			out= new PrintWriter(new OutputStreamWriter(aClient.getOutputStream()), true);
			out.println("220 Simple mail server");
		}
		catch (IOException ioe) {
			System.out.println("SMTPThread: IO Initialization Error!");
		}
	}

	/**
	 *
	 */
  public void run() {
		try {
		  while (true) {
				command= in.readLine();

				//Check If Client Quitting...
				if (command.toUpperCase().equals("QUIT")) {
					out.println("221 Bye");
					in.close();
					out.close();
					client.close();
					break;
				}

				switch (currentState) {
					case 0: {  //Initial state
						StringTokenizer tok= new StringTokenizer(command, " ");
						token= tok.nextToken();
						if (token.toUpperCase().equals("HELO")) {
						  if (tok.countTokens()== 1) {
							  out.println("250 Welcome "+ tok.nextToken());
								currentState= 1;
							}
							else
								out.println("500 Invalid command");
						}
						else if ((token.toUpperCase().equals("MAIL")) || (token.toUpperCase().equals("RCPT")) || (token.toUpperCase().equals("DATA")))
							out.println("503 Invalid state");
						else
							out.println("500 Invalid command");
					} break;
					case 1: {  //HELO state
						StringTokenizer tok= new StringTokenizer(command, " ");
						token= tok.nextToken();
						if (token.toUpperCase().equals("MAIL")) {
							if ((tok.nextToken().toUpperCase().equals("FROM:")) && (tok.countTokens()== 1)) {
								token= tok.nextToken();
								if (token.charAt(0)== '<') {
									if (token.charAt(token.length()- 1)== '>') {
										out.println("250 "+ token.substring(1, token.length()-1)+ " Sender OK");
										currentState= 2;
									}
									else
									  out.println("500 Invalid command");
								}
								else if (token.charAt(token.length()-1)== '>')
									out.println("500 Invalid command");
								else {
									out.println("250 "+ token+" Sender OK");
									currentState= 2;
								}
							}
							else
							  out.println("500 Invalid command");
						}
						else if ((token.toUpperCase().equals("HELO")) || (token.toUpperCase().equals("RCPT")) || (token.toUpperCase().equals("DATA")))
							out.println("503 Invalid state");
						else
							out.println("500 Invalid command");
					} break;
					case 2: {  //MAIL FROM: state
						StringTokenizer tok= new StringTokenizer(command, " ");
						token= tok.nextToken();
						if (token.toUpperCase().equals("RCPT")) {
							if ((tok.nextToken().toUpperCase().equals("TO:")) && (tok.countTokens()== 1)) {
								token= tok.nextToken();
								if (token.charAt(0)== '<') {
									if (token.charAt(token.length()- 1)== '>') {
										userName= token.substring(1, token.length()-1);
										userIndex= ref.userExists(userName);
										if (userIndex!= -1) {
											out.println("250 "+ userName+ " Recipient OK");
											currentState= 3;
										}
										else
											out.println("550 "+ userName+ " Recipient not found on this server");
									}
									else
									  out.println("500 Invalid command");
								}
								else if (token.charAt(token.length()-1)== '>')
									out.println("500 Invalid command");
								else {
									userName= token.substring(0, token.length());
									userIndex= ref.userExists(userName);
									if (userIndex!= -1) {
										out.println("250 "+ userName+ " Recipient OK");
										currentState= 3;
									}
									else
										out.println("550 "+ userName+ " Recipient not found on this server");
								}
							}
							else
							  out.println("500 Invalid command");
						}
						else if ((token.toUpperCase().equals("HELO")) || (token.toUpperCase().equals("MAIL")) || (token.toUpperCase().equals("DATA")))
							out.println("503 Invalid state");
						else
							out.println("500 Invalid command");
					} break;
					case 3: {  //RCPT TO: state
						StringTokenizer tok= new StringTokenizer(command, " ");
						token= tok.nextToken();
						if ((token.toUpperCase().equals("DATA"))) {
							if (tok.hasMoreTokens())
								out.println("500 Invalid command");
							else {
								out.println("354 Enter mail, end with \".\" on a line by itself");
								currentState= 4;
							}
						}
						else if ((token.toUpperCase().equals("HELO")) || (token.toUpperCase().equals("MAIL")) || (token.toUpperCase().equals("RCPT")))
							out.println("503 Invalid state");
						else
							out.println("500 Invalid command");
					} break;
					case 4: {  //DATA state
						if (command.charAt(0)== '.') {
							//Message ended, save it (user-synchronized)
							synchronized(ref.locks[userIndex]) {
								File userDir= new File("./users/"+ userName);
								int numMessages= userDir.list().length;
								PrintWriter msgWriter= new PrintWriter(new FileWriter("./users/"+ userName+ "/"+ (numMessages+1)+ ".m"), true);
								msgLines= msgLines.substring(0, msgLines.length()-1);
								msgWriter.println(msgLines);
								msgWriter.close();
								out.println("250 Message delivered");
								currentState= 1;
							}
						}
						else ;
							msgLines= msgLines+ command+ "\n";
					} break;
				}
			}
		}
		catch (IOException ioe) {
			System.out.println("SMTPThread: IO Error");
		}
	}
}