

package fmi.ipj.proxy;


import java.net.Socket;
import java.net.InetAddress;

import java.io.InputStream;
import java.io.OutputStream;

import fmi.ipj.proxy.constants.Constants;

import fmi.ipj.proxy.request.HTTPRequest;
import fmi.ipj.proxy.request.HTTPRequestProcessor;
import fmi.ipj.proxy.request.InvalidRequestException;
import fmi.ipj.proxy.request.ValidateRequestException;

import fmi.ipj.proxy.connector.StreamInitializer;
import fmi.ipj.proxy.connector.RemoteHostConnector;
import fmi.ipj.proxy.connector.ConnectionFailedException;

import fmi.ipj.proxy.response.HTTPResponseCreater;

import fmi.ipj.proxy.log.Log;


public class HTTPProxyThread implements Runnable
{

	private StreamInitializer aoClientStreamsInitializer 		= null;
	private RemoteHostConnector aoRemoteHostStreamsInitializer 	= null;

	private Socket aoClientSocket								= null;
	private String aoClientString								= null;
	private InputStream aoClientInputStream				 		= null;
	private OutputStream aoClientOutputStream 					= null;
	private String aoClientRequest								= Constants.EMPTY_VALUE;
	
	private Socket aoRemoteServerSocket							= null;	
	private String aoRemoteServerString							= null;
	private InputStream aoRemoteServerInputStream 				= null;
	private OutputStream aoRemoteServerOutputStream 			= null;
	private String aoResponseFromServer							= Constants.EMPTY_VALUE;
	
	private byte aDataBuffer[]									= new byte[ Constants.aBufferSize ];
	
	
	public HTTPProxyThread( Socket poClient )
	{
		aoClientSocket = poClient;
	}
	
	
	public void run()
	{
		dataTransfer();
	}


	private void dataTransfer()
	{
		boolean lClientStreamsIsOK 				= false;
		boolean lRemoteServerStreamsIsOK 		= false;
		boolean lRequestSendToServerIsOK 		= false;
		boolean lResponseSendToClientIsOK 		= false;
		
		try
		{
			initClientStreams();
			lClientStreamsIsOK = true;
		}
		catch( Exception e )
		{
			processError( e, false, false );
		}
		
		if ( lClientStreamsIsOK )
		{
			try
			{
				initRemoteServerStreams();
				lRemoteServerStreamsIsOK = true;
			}
			catch( Exception e )
			{
				processError( e, true, true );
			}
		}
		
		if ( lClientStreamsIsOK && lRemoteServerStreamsIsOK )
		{
			try
			{
				sendRequestToServer();
				lRequestSendToServerIsOK = true;
				logMessage( Constants.REQUEST_SEND_TO_SERVER + Constants.NEW_LINE + Constants.OPEN_BRACKET + Constants.NEW_LINE + aoClientRequest + Constants.NEW_LINE + Constants.CLOSE_BRACKET );				
			}
			catch( Exception e )
			{
				processError( e, true, true );
			}
			
			if ( lRequestSendToServerIsOK )
			{
				try
				{
					receivedResponseFromServerAndSendToClient();					
					lResponseSendToClientIsOK = true;
				}
				catch( Exception e )
				{
					processError( e, true, true );
				}
			}
			
			if ( lResponseSendToClientIsOK )
			{
				closeRemoteServerStreams();
				closeClientStreams();
				logMessage( Constants.OK );
			}
		}
	}

	
	private void initClientStreams()
		throws ConnectionFailedException
	{
			//Init client streams start
			aoClientStreamsInitializer 	= new StreamInitializer( aoClientSocket );
			
			aoClientString 				= aoClientStreamsInitializer.getHostName() + Constants.OPEN_BRACKET
										+ aoClientStreamsInitializer.getHostIP() + Constants.HOST_PORT_SEPARATER
										+ aoClientStreamsInitializer.getHostPort() + Constants.CLOSE_BRACKET;
			aoClientInputStream 				= aoClientStreamsInitializer.getInputStream();
			aoClientOutputStream 				= aoClientStreamsInitializer.getOutputStream();
			//Init client streams end
	}
	

	private void initRemoteServerStreams()
		throws ConnectionFailedException, InvalidRequestException, ValidateRequestException
	{
			//Init remote server streams start
			HTTPRequestProcessor lClientRequestProcessor 	= new HTTPRequestProcessor();
			HTTPRequest lRequest 							= lClientRequestProcessor.getHTTPRequest( aoClientInputStream );
			aoClientRequest									= lRequest.toString();
			logMessage( Constants.REQUEST_READ_FROM_CLIENT + Constants.NEW_LINE + Constants.OPEN_BRACKET + Constants.NEW_LINE + aoClientRequest + Constants.NEW_LINE + Constants.CLOSE_BRACKET );
			
			aoRemoteHostStreamsInitializer					= new RemoteHostConnector( lRequest );
			aoRemoteServerSocket 							= aoRemoteHostStreamsInitializer.getRemoteServerSocket();
			aoRemoteServerString 							= aoRemoteHostStreamsInitializer.getRemoteHostName() + Constants.OPEN_BRACKET
															+ aoRemoteHostStreamsInitializer.getRemoteHostIP() + Constants.HOST_PORT_SEPARATER
															+ aoRemoteHostStreamsInitializer.getRemoteHostPort() + Constants.CLOSE_BRACKET;
			aoRemoteServerInputStream 							= aoRemoteHostStreamsInitializer.getInputStreamFromRemoteHost();
			aoRemoteServerOutputStream 							= aoRemoteHostStreamsInitializer.getOutputStreamToRemoteHost();
			//Init remote server streams end
	}

	
	private void sendRequestToServer()
		throws ConnectionFailedException
	{
		try
		{
			aoRemoteServerOutputStream.write( aoClientRequest.getBytes() );
			aoRemoteServerOutputStream.flush();
		}
		catch( java.io.IOException ioe )
		{
			throw new ConnectionFailedException( this, Constants.CONNECTION_TO_HOST_LOST );
		}
	}
	
	
	private void receivedResponseFromServerAndSendToClient()
		throws ConnectionFailedException
	{
		try
		{	
			int	lDataSize = -1;
					
			//Read from server and send to client.
			while ( ( lDataSize = aoRemoteServerInputStream.read( aDataBuffer ) ) != -1 )
			{
				logMessage( Constants.RESPONSE_READ_FROM_SERVER + Constants.NEW_LINE + Constants.OPEN_BRACKET + Constants.NEW_LINE + new String( aDataBuffer, 0, lDataSize ) + Constants.NEW_LINE + Constants.CLOSE_BRACKET );
				aoClientOutputStream.write( aDataBuffer, 0, lDataSize );
				aoClientOutputStream.flush();
				logMessage( Constants.RESPONSE_SEND_TO_CLIENT + Constants.NEW_LINE + Constants.OPEN_BRACKET + Constants.NEW_LINE + new String( aDataBuffer, 0, lDataSize ) + Constants.NEW_LINE + Constants.CLOSE_BRACKET );
			}
		}
		catch( java.io.IOException ioe )
		{
			throw new ConnectionFailedException( this, Constants.CONNECTION_TO_HOST_LOST );
		}
	}
	
	
	private void sendToClientErrorMessage( Exception poException )
	{
		HTTPResponseCreater loHTTPResponseCreater 	= new HTTPResponseCreater();
		String loMessage 							= loHTTPResponseCreater.createResponseForError( poException );
		logMessage( Constants.RESPONSE_SEND_TO_CLIENT + Constants.NEW_LINE + Constants.OPEN_BRACKET + Constants.NEW_LINE + loMessage + Constants.NEW_LINE + Constants.CLOSE_BRACKET );		

		try
		{
			aoClientOutputStream.write( loMessage.getBytes() );
			aoClientOutputStream.flush();
		}
		catch( java.io.IOException ioe )
		{
			// Do nothing!
		}
	}
	
	
	private void processError( Exception poException, boolean pSendToClient, boolean pCloseServerStreams )
	{
//		poException.printStackTrace();
		logMessage( poException.getMessage() );
		
		if ( pCloseServerStreams )
		{
			closeRemoteServerStreams();
		}
		
		if ( pSendToClient )
		{
			sendToClientErrorMessage( poException );
		}
		
		closeClientStreams();
	}
	
	
	private void logMessage( String poMessage )
	{
		String lMessage = Constants.PROCESS_ID + Thread.currentThread().hashCode()
						+ Constants.CLIENT + aoClientString
						+ Constants.SERVER + aoRemoteServerString
						+ Constants.SPACE + poMessage;
						
		Log.logMessage( lMessage );
	}
	
	
	private void closeClientStreams()
	{
		try
		{
			if ( aoClientInputStream != null )
			{
				aoClientInputStream.close();
			}
		}
		catch( java.io.IOException ioe )
		{
			// Do nothing!
		}

		try
		{
			if ( aoClientOutputStream != null )
			{		
				aoClientOutputStream.close();
			}
		}
		catch( java.io.IOException ioe )
		{
			// Do nothing!
		}
		
		try
		{
			if ( aoClientSocket != null )
			{		
				aoClientSocket.close();
			}
		}
		catch( java.io.IOException ioe )
		{
			// Do nothing!
		}		
	}
	
	
	private void closeRemoteServerStreams()
	{
		try
		{
			if ( aoRemoteServerInputStream != null )
			{		
				aoRemoteServerInputStream.close();
			}
		}
		catch( java.io.IOException ioe )
		{
			// Do nothing!
		}
		
		try
		{
			if ( aoRemoteServerOutputStream != null )
			{		
				aoRemoteServerOutputStream.close();
			}
		}
		catch( java.io.IOException ioe )
		{
			// Do nothing!
		}
		
		try
		{
			if ( aoRemoteServerSocket != null )
			{		
				aoRemoteServerSocket.close();
			}
		}
		catch( java.io.IOException ioe )
		{
			// Do nothing!
		}		
	}	
}
