

package fmi.ipj.proxy.request;


import java.io.InputStream;
//import java.io.FileInputStream;
//import java.io.InputStreamReader;
//import java.io.BufferedReader;

import fmi.ipj.proxy.constants.Constants;
import fmi.ipj.proxy.log.Log;


public class HTTPRequestParser
{
	private StringBuffer loLineBuffer = new StringBuffer( 256 );
	
	
	//End of line have three case:
	// \r\n - WIN
	// \n\r - MAC OS
	// \n   - LINUX
	private String readLine( InputStream poClientIS ) throws java.io.IOException
	{
		loLineBuffer.delete( 0, loLineBuffer.length() );
		boolean loEndOfLineReceived	= true;
		
		char loTempCharachter;
		
		while ( loEndOfLineReceived )
		{
			loTempCharachter = (char) poClientIS.read();
			
			if ( Constants.NEW_LINE_CHAR == loTempCharachter )
			{
				loEndOfLineReceived= false;
				continue;
			}
			
			if ( ! ( Constants.CARRIAGE_RETURN == loTempCharachter ) )
			{
				loLineBuffer.append( loTempCharachter );
			}
		}
		
		return loLineBuffer.toString();
	}
	
	
	public HTTPRequest getHTTPRequest( InputStream poClientIS ) throws fmi.ipj.proxy.request.InvalidRequestException
	{
		HTTPRequest loHTTPRequest 	= new HTTPRequest();
		String loTempLine 			= Constants.EMPTY_VALUE;
		boolean lIsFirstLine 		= true;
		
		try
		{
			while ( !Constants.EMPTY_VALUE.equals( loTempLine = readLine( poClientIS ) ) )
			{	
				if ( lIsFirstLine )
				{
					parseRequestFirstLine( loTempLine, loHTTPRequest );
					lIsFirstLine = false;
				}
				else
				{
					parseRequestParameters( loTempLine, loHTTPRequest );			
				}
			}
		}
		catch( java.io.IOException ioe )
		{
			throw new InvalidRequestException( this, Constants.IO_ERROR + Constants.SPACE + ioe.getLocalizedMessage() );
		}

		return loHTTPRequest;
	}
	
	
	private void parseRequestFirstLine( String poTempLine, HTTPRequest poHTTPRequest ) throws fmi.ipj.proxy.request.InvalidRequestException
	{
		String loTempLine = poTempLine;
		while ( loTempLine.indexOf( Constants.SPACE ) == 0 ) //Escape spaces in start of request.
		{
			loTempLine = loTempLine.substring( 1, loTempLine.length() );
		}
		
		int lIndexOfSpace = loTempLine.indexOf( Constants.SPACE );
		if ( lIndexOfSpace != -1 )
		{
			String loRequestName = loTempLine.substring( 0, lIndexOfSpace );
			if ( !Constants.isRequestAllowed( loRequestName ) )
			{
				throw new InvalidRequestException( this, Constants.REQUEST_NAME_NOT_ALLOWED );			
			}
			
			if ( Constants.GET_REQUEST_NAME.toUpperCase().equals( loRequestName ) )
			{
				poHTTPRequest.setRequestName( loRequestName );
				
				loTempLine = loTempLine.substring( lIndexOfSpace, loTempLine.length() );			
				while ( loTempLine.indexOf( Constants.SPACE ) == 0 ) //Escape spaces in start of request.
				{
					loTempLine = loTempLine.substring( 1, loTempLine.length() );
				}
				
				int lIndexOfProtocol = loTempLine.indexOf( Constants.DEFAULT_PROTOCOL.toUpperCase() );
				if ( lIndexOfProtocol != -1 )
				{
					String lProtocol 	= loTempLine.substring( lIndexOfProtocol, loTempLine.length() );
					lProtocol 			= lProtocol.trim();
					int lIndexOfProtocolVersionSeparater = lProtocol.indexOf( Constants.PROTOCOL_VERSION_SEPARATER );
					if ( lIndexOfProtocolVersionSeparater != -1 )
					{
						String lProtocolType 	= lProtocol.substring( 0, lIndexOfProtocolVersionSeparater );
						poHTTPRequest.setProtocol( lProtocolType );
						String lProtocolVersion = lProtocol.substring( lIndexOfProtocolVersionSeparater + 1, lProtocol.length() );
						poHTTPRequest.setProtocolVersion( lProtocolVersion );
					}
					else
					{
						throw new InvalidRequestException( this, Constants.VERSION_NOT_FOUND );				
					}
					
					loTempLine = loTempLine.substring( 0, lIndexOfProtocol );
					loTempLine = loTempLine.trim();
					parseURL( loTempLine, poHTTPRequest );
				}
				else
				{
					throw new InvalidRequestException( this, Constants.PROTOCOL_NOT_FOUND );
				}
			}
		}
		else
		{
			throw new InvalidRequestException( this, Constants.SPACE_NOT_FOUND );
		}
	}
	
	
	private void parseRequestParameters( String poTempLine, HTTPRequest poHTTPRequest ) throws fmi.ipj.proxy.request.InvalidRequestException
	{
		String loTempLine = poTempLine;
		while ( loTempLine.indexOf( Constants.SPACE ) == 0 ) //Escape spaces in start of line.
		{
			loTempLine = loTempLine.substring( 1, loTempLine.length() );
		}
		loTempLine = loTempLine.trim();
		
		int lIndexOfKeyValueSeparater = loTempLine.indexOf( Constants.KEY_VALUE_SEPARATER );
		if ( lIndexOfKeyValueSeparater != -1 )
		{
			String lKey 	= loTempLine.substring( 0, lIndexOfKeyValueSeparater );
			String lValue 	= loTempLine.substring( lIndexOfKeyValueSeparater + 1, loTempLine.length() );
			
			while ( lValue.indexOf( Constants.SPACE ) == 0 )
			{
				lValue = lValue.substring( 1, lValue.length() );
			}
			lValue = lValue.trim();
			
			poHTTPRequest.addProperty( lKey, lValue );
			if ( Constants.HOST_KEY.equals( lKey ) )
			{				
				int lIndexOfHostPortSeparater = lValue.indexOf( Constants.HOST_PORT_SEPARATER );
				if ( lIndexOfHostPortSeparater != -1 ) //Port found.
				{
					String lPort = lValue.substring( lIndexOfHostPortSeparater + 1, lValue.length() );
					if ( !poHTTPRequest.aIsPortSet )
					{
						poHTTPRequest.setPort( lPort );
						try
						{
							int lPortInt = Integer.parseInt( lPort );
							poHTTPRequest.setPortInt( lPortInt );
						}
						catch( NumberFormatException nfe )
						{
							throw new InvalidRequestException( this, Constants.BAD_PORT );
						}
					}
					
					lValue = lValue.substring( 0, lIndexOfHostPortSeparater );
				}
				
				if ( !Constants.EMPTY_VALUE.equals( lValue) && !poHTTPRequest.aIsHostSet )
				{
					poHTTPRequest.setHost( lValue );
				}				
			}
		}
		else
		{
			throw new InvalidRequestException( this, Constants.WRONG_FORMAT_FOR_KEY_VALUE );
		}
	}
	
	
	private void parseURL( String poURL, HTTPRequest poHTTPRequest ) throws fmi.ipj.proxy.request.InvalidRequestException
	{
		String poTempURL 			= poURL.replace( '\\', Constants.PATH_SEPARATER_CHAR );
//		System.out.println( "parseURL( " + poURL + ", poHTTPRequest ) invocated." );
				
		int lIndexOfProtocolSuffix 	= poTempURL.indexOf( Constants.PROTOCOL_SUFFIX );
		if ( lIndexOfProtocolSuffix != -1 )	//Protocol name exists!
		{
			//Get and cut protocol.
			String lProtocol = poTempURL.substring( 0, lIndexOfProtocolSuffix );
			poHTTPRequest.setProtocolFromURL( lProtocol );
			
			poTempURL = poTempURL.substring( lIndexOfProtocolSuffix + Constants.PROTOCOL_SUFFIX.length(), poTempURL.length() );
		}
		
		int lIndexOfHostSuffix 	= poTempURL.indexOf( Constants.HOST_SUFFIX );
		if ( lIndexOfHostSuffix != -1 ) // Try to get host name.
		{
			String lHost = poTempURL.substring( 0, lIndexOfHostSuffix );
			if ( !Constants.EMPTY_VALUE.equals( lHost) ) // Host found!
			{
				int lIndexOfHostPortSeparater = lHost.indexOf( Constants.HOST_PORT_SEPARATER );
				if ( lIndexOfHostPortSeparater != -1 ) //Port found.
				{
					String lPort = lHost.substring( lIndexOfHostPortSeparater + 1, lHost.length() );
					poHTTPRequest.setPort( lPort );
					try
					{
						int lPortInt = Integer.parseInt( lPort );
						poHTTPRequest.setPortInt( lPortInt );
					}
					catch( NumberFormatException nfe )
					{
						throw new InvalidRequestException( this, Constants.BAD_PORT );
					}
					
					lHost = lHost.substring( 0, lIndexOfHostPortSeparater );
				}
				
				if ( !Constants.EMPTY_VALUE.equals( lHost) )
				{
					poHTTPRequest.setHost( lHost );
				}

				poTempURL = poTempURL.substring( lIndexOfHostSuffix, poTempURL.length() );				
			}
			poTempURL = poTempURL.substring( Constants.HOST_SUFFIX.length(), poTempURL.length() );
			poHTTPRequest.setPath( poTempURL );
		}
		else
		{
			String lHost = poTempURL;
			if ( !Constants.EMPTY_VALUE.equals( lHost) ) // Host found!
			{
				int lIndexOfHostPortSeparater = lHost.indexOf( Constants.HOST_PORT_SEPARATER );
				if ( lIndexOfHostPortSeparater != -1 ) //Port found.
				{
					String lPort = lHost.substring( lIndexOfHostPortSeparater + 1, lHost.length() );
					poHTTPRequest.setPort( lPort );
					lHost = lHost.substring( 0, lIndexOfHostPortSeparater );
				}
				
				if ( !Constants.EMPTY_VALUE.equals( lHost) )
				{
					poHTTPRequest.setHost( lHost );
				}				
			}		
		}
	}
	
	
	public static void main( String args[] )
	{
//		HTTPRequestParser parser = new HTTPRequestParser();
//		try
//		{
//			FileInputStream fis = new FileInputStream( "./request.txt" );
//			BufferedReader d = new BufferedReader( new InputStreamReader( fis ) );		
//		
//			HTTPRequest request = parser.getHTTPRequest( d );
//			System.out.println( "'" + request + "'" );
//		}
//		catch( Exception e)
//		{
//			e.printStackTrace();
//		}
	}
}
