/*
	UDP Chat Client
	Internet Programming with Java Course
	
	(c) 2002 Martin Valkanov
		
	The UDPClientListener thread receives messages from the server
	or other clients and displays them to the console.
*/

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.io.PrintWriter;
import java.io.IOException;

public class UDPClientListener extends Thread
{
	// Set receiving buffer size. If message is larger it will be truncated by
	// the datagram socket.
	private final static int BUF_SIZE = 512;
	private DatagramSocket mSock;
	
	// Received messages will be written here.
	private PrintWriter mOut; 
	
	public UDPClientListener( DatagramSocket sock, PrintWriter out )
	{
		mSock = sock;
		mOut = out;
	}
	
	public void run()
	{
		byte buf[] = new byte[BUF_SIZE];
		DatagramPacket dp;
		String sMsg;

		// Run until the user types quit the primary thread instructs us to
		// stop receiving.		
		while ( !isInterrupted() )
		{
			// Create a blank datagram packet
			dp = new DatagramPacket( buf, buf.length );
			
			try
			{
				// Receive a message
				mSock.receive( dp );
				sMsg = new String( dp.getData(), dp.getOffset(), dp.getLength() );
				
				// Print the source of the message
				if ( !sMsg.startsWith( "Received message from " ) )
					mOut.print( "Received message from " + dp.getAddress().getHostAddress() + ":" + dp.getPort() + ": " );
					
				// Print the actual contents of the message
				mOut.println(  sMsg );
				mOut.flush();
			}
			catch ( IOException exIO )
			{
				if ( isInterrupted() ) break;
				System.err.println( "IOException: " + exIO );
				exIO.printStackTrace();
			}
		}
	}
};