/*
	UDP Chat Server
	Internet Programming with Java Course
	
	(c) 2002 Martin Valkanov

	ClientServer is a message delivering thread.
	Incoming messages from the ServerDispatcher are stored in a queue.
*/

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.util.Vector;
import java.io.IOException;
import java.net.SocketException;

public class ClientSender extends Thread
{
	ClientInfo mClientInfo;
	DatagramSocket mSocket;
	Vector mMessageQueue = new Vector();
	private boolean mNotTerminated = true;
	
	ClientSender( ClientInfo client )
	{
		mClientInfo = client;
	}
	
	// ServerDispatcher pushes another message to the queue.
	void queueMessage( String sMsg )
	{
		synchronized ( mMessageQueue )
		{
			mMessageQueue.add( sMsg );
			mMessageQueue.notify();
		}
	}
	
	// Retrieve a message from the queue.
	// If it is empty wait.
	String getNextMessage() throws InterruptedException
	{
		while ( mMessageQueue.isEmpty() )
			synchronized ( mMessageQueue ) { mMessageQueue.wait(); }
			
		String sMsg;
		
		synchronized ( mMessageQueue )
		{
			sMsg = (String) mMessageQueue.elementAt(0);
			mMessageQueue.remove(0);
		}
		
		return sMsg;
	}
	
	void sendMessage( String sMsg )
	{
		byte buf[] = sMsg.getBytes();
		
		DatagramPacket dp = new DatagramPacket( buf, buf.length, mClientInfo.mAddr, mClientInfo.mPort );
		try
		{
			mSocket.send(dp);
		}
		catch ( IOException exIO )
		{
			System.err.println( "Unable to send packet: " + exIO );
			exIO.printStackTrace();
		}
	}
	
	public void terminate()
	{
		mNotTerminated = false;
	}
	
	public void run()
	{
		try
		{
			mSocket = new DatagramSocket();
		}
		catch ( SocketException exSock )
		{
			System.err.println( "Socket exception: " + exSock );
			exSock.printStackTrace();
			return;
		}
		
		// Message pump
		try
		{
			while ( mNotTerminated )
			{
				String sMsg = getNextMessage();
				sendMessage( sMsg );
			}
		}
		catch ( InterruptedException exIntr ) {}
	}
};