package smtp_and_pop3;

/**
 * <p>Title: inet java 1.4</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Milena Iordanova
 * @version 1.0
 */
import smtp_and_pop3.*;
import java.util.*;

/**
 * This class manages the mailboxes. It is used to manage users
 * (creation / removal), as well as to add a message into the mailbox of a
 * user.
 * <p>Title: inet java 1.4</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Milena Iordanova
 * @version 1.0
 */
public class MailBoxManager {
  private ArrayList m_MailBoxes; // Stores MailBox
  private ArrayList m_Passwords; // Stores String
  private ArrayList m_Usernames; // Stores String
  private ArrayList m_Used;      // Stores Boolean
  private ArrayList m_Queues;    // Stores ArrayList that store String[]

  public MailBoxManager() {
    m_MailBoxes = new ArrayList();
    m_Passwords = new ArrayList();
    m_Usernames = new ArrayList();
    m_Used = new ArrayList();
    m_Queues = new ArrayList();
  }

  /**
   * Adds a user to the system
   * @param strUsername Alias for the new user
   * @param strPassword New user's password
   */
  public synchronized void AddUser(String strUsername, String strPassword) {
    m_Usernames.add(strUsername);
    m_Passwords.add(strPassword);
    m_MailBoxes.add(new MailBox());
    m_Used.add(new Boolean(false));
    m_Queues.add(new ArrayList());
  }

  /**
   * Returns the mailbox for a user. This method can also be used for
   * authentication. After the mailbox is no longer needed, it should be
   * closed with CloseMailBox().
   * @param strUsername Username
   * @param strPassword Password
   * @return The mailbox of the user Mailbox, if it is available
   * @throws Exception The description of this exception can be used to
   * find out whether the user is already logged in, there is no such user,
   * or the password is not correct.
   */
  public synchronized MailBox GetMailBox(String strUsername, String strPassword) throws Exception {
    int iNumMailBoxes = m_Usernames.size();
    for(int i = 0; i < iNumMailBoxes; i++) {
      if(strUsername.equals(m_Usernames.get(i))) {
        if(strPassword.equals(m_Passwords.get(i))) {
          if(!((Boolean)m_Used.get(i)).booleanValue()) {
            m_Used.set(i, new Boolean(true));
            return (MailBox)m_MailBoxes.get(i);
          }

          throw new Exception("Already logged in");
        }

        throw new Exception("Wrong password");
      }
    }
    throw new Exception("Wrong username");
  }

  /**
   * Releases a mailbox previously returned by GetMailBox(). If any messages
   * are waiting in the queue, they are transfered into the mailbox.
   * @param mb Mailbox previously returned by GetMailBox().
   */
  public synchronized void CloseMailBox(MailBox mb) {
    m_Used.set(m_MailBoxes.indexOf(mb), new Boolean(false));

    ArrayList queue = (ArrayList)m_Queues.get(m_MailBoxes.indexOf(mb));
    while(!queue.isEmpty()) {
      mb.AddMessage((String[])queue.get(0));
      queue.remove(0);
    }
  }

  /**
   * Removes a user from the system.
   * @param strUsername The name of the user to be removed
   */
  public synchronized void RemoveUser(String strUsername) {
    int iNumUsers = m_Usernames.size();
    for(int i = 0; i < iNumUsers; i++) {
      if(strUsername.equals(m_Usernames.get(i))) {
        m_MailBoxes.remove(i);
        m_Usernames.remove(i);
        m_Passwords.remove(i);
        m_Used.remove(i);
        m_Queues.remove(i);
        return;
      }
    }
  }

  /**
   * Checks if a user exists in the system.
   * @param strUsername Username of the user
   * @return Whether the user exists in the system
   */
  public synchronized boolean UserExists(String strUsername) {
    int iNumUsers = m_Usernames.size();
    for(int i = 0; i < iNumUsers; i++) {
      if(strUsername.equals(m_Usernames.get(i)))
        return true;
    }

    return false;
  }

  /**
   * Adds a message to the mailbox of a given user. By specification, the
   * mailbox should not be modified during a POP3 session. That is why this
   * method checks if the mailbox is currently open by a POP3 session, and
   * in such case it adds the message to a queue. When the POP3 session
   * closes, the messages from the queue are automatically moved into the
   * mailbox.
   * @param strUsername Username of the user receiving the message
   * @param message The message to be added to the mailbox
   */
  public synchronized void AddMessage(String strUsername, String[] message) {
    int iNumUsers = m_Usernames.size();
    for(int i = 0; i < iNumUsers; i++) {
      if(strUsername.equals(m_Usernames.get(i))) {
        if(((Boolean)m_Used.get(i)).booleanValue())
          ((ArrayList)m_Queues.get(i)).add(message);
        else
          ((MailBox)m_MailBoxes.get(i)).AddMessage(message);

        return;
      }
    }
  }
}