package smtp_and_pop3;

/**
 * <p>Title: inet java 1.4</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Milena Iordanova
 * @version 1.0
 */

import smtp_and_pop3.*;
import java.net.*;
import java.io.*;

/**
 * This class handles the POP3 sessions
 * <p>Title: inet java 1.4</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Milena Iordanova
 * @version 1.0
 */
public class POP3Instance extends Thread {

  Socket m_Socket;
  String m_strUsername = null;
  BufferedReader m_brInput;
  BufferedWriter m_bwOutput;
  MailBox m_MailBox = null;
  boolean m_bQuit = false;


  public POP3Instance(Socket s) throws java.io.IOException {
    m_Socket = s;

    m_brInput = new BufferedReader(new InputStreamReader(m_Socket.getInputStream()));
    m_bwOutput = new BufferedWriter(new OutputStreamWriter(m_Socket.getOutputStream()));
  }

  /**
   * Sends a line of text over the socket
   * @param s The text to be sent
   * @throws IOException When I/O error occurs
   */
  private void WriteLine(String s) throws java.io.IOException {
    m_bwOutput.write(s);
    m_bwOutput.newLine();
    m_bwOutput.flush();
  }

  /**
   * Handles the POP3 session
   */
  public void run() {
    try {
      WriteLine("+OK POP3 server ready");

      String strCommand;

      while(!m_bQuit && ((strCommand = m_brInput.readLine()) != null)) {
        if(strCommand.startsWith("USER "))
          CmdUser(strCommand.substring(5));
        else if(strCommand.startsWith("PASS "))
          CmdPass(strCommand.substring(5));
        else if(strCommand.equals("LIST"))
          CmdList();
        else if(strCommand.startsWith("RETR "))
          CmdRetr(strCommand.substring(5));
        else if(strCommand.equals("QUIT"))
          CmdQuit();
        else {
          WriteLine("-ERR Invalid command");
        }
      }
    }
    catch(java.io.IOException e) {
    }

    if(m_MailBox != null) {
      AppClass.appClass1.m_MailBoxManager.CloseMailBox(m_MailBox);
    }

    try {
      m_Socket.close();
    }
    catch(java.io.IOException e) {
    }
  }

  /**
   * Processes the USER command
   * @param strUsername Parameter supplied to the USER command
   * @throws IOException When I/O error occurs
   */
  private void CmdUser(String strUsername) throws java.io.IOException {
    m_strUsername = strUsername;

    WriteLine("+User OK, please send your password");
  }

  /**
   * Processes the PASS command
   * @param strPass Parameter supplied to the PASS command
   * @throws IOException When I/O error occurs
   */
  private void CmdPass(String strPass) throws java.io.IOException {
    try {
      if(m_strUsername == null)
        throw new Exception("Password with no user!!!");

      m_MailBox = AppClass.appClass1.m_MailBoxManager.GetMailBox(m_strUsername, strPass);
      WriteLine("+OK");
    }
    catch(Exception e) {
      WriteLine("-ERR Invalid username/password");
    }
  }

  /**
   * Processes the LIST command
   * @throws IOException When I/O error occurs
   */
  private void CmdList() throws java.io.IOException {
    if(m_MailBox == null) {
      WriteLine("-ERR Invalid state");
      return;
    }

    String[] strMessageList = m_MailBox.List();
    WriteLine("+OK " + Integer.toString(strMessageList.length) + " messages");

    for(int i = 0; i < strMessageList.length; i++) {
      WriteLine(Integer.toString(i + 1) + " " + strMessageList[i]);
    }

    WriteLine(".");
  }

  /**
   * Processes the RETR command
   * @param strMessageNum Parameter supplied to the RETR command
   * @throws IOException When I/O error occurs
   */
  private void CmdRetr(String strMessageNum) throws java.io.IOException {
    if(m_MailBox == null) {
      WriteLine("-ERR Invalid state");
      return;
    }

    int iMessageNum;

    try {
      iMessageNum = Integer.parseInt(strMessageNum);
    }
    catch(Exception e) {
      WriteLine("-ERR No such message");
      return;
    }

    if((iMessageNum < 1) || (iMessageNum > m_MailBox.GetMessageCount())) {
      WriteLine("-ERR No such message");
      return;
    }

    String[] message = m_MailBox.Retrieve(iMessageNum - 1);

    WriteLine("+OK message " + strMessageNum);

    for(int i = 0; i < message.length; i++)
      WriteLine(message[i]);

    WriteLine(".");
  }

  /**
   * Processes the QUIT command
   * @throws IOException when I/O error occurs
   */
  private void CmdQuit() throws java.io.IOException {
    WriteLine("+OK Bye");
    m_bQuit = true;
  }
}