package smtp_and_pop3;

/**
 * <p>Title: inet java 1.4</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Milena Iordanova
 * @version 1.0
 */
import smtp_and_pop3.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * This class handles the SMTP sessions
 * <p>Title: inet java 1.4</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Milena Iordanova
 * @version 1.0
 */
public class SMTPInstance extends Thread {
  Socket m_Socket;
  BufferedReader m_brInput;
  BufferedWriter m_bwOutput;
  String m_strSenderEmail = null;
  String m_strRecipientEmail = null;
  boolean m_bQuit = false;

  public SMTPInstance(Socket s) throws java.io.IOException {
    m_Socket = s;

    m_brInput = new BufferedReader(new InputStreamReader(m_Socket.getInputStream()));
    m_bwOutput = new BufferedWriter(new OutputStreamWriter(m_Socket.getOutputStream()));
  }

  /**
   * Sends a line of text over the socket
   * @param s The text to be sent
   * @throws IOException When I/O error occurs
   */
  private void WriteLine(String s) throws java.io.IOException {
    m_bwOutput.write(s);
    m_bwOutput.newLine();
    m_bwOutput.flush();
  }

  /**
   * Handles the SMTP session
   */
  public void run() {
    try {
      WriteLine("220 Simple mail server");

      String strCommand;

      while(!m_bQuit && ((strCommand = m_brInput.readLine()) != null)) {
        if(strCommand.startsWith("HELO "))
          CmdHelo(strCommand.substring(5));
        else if(strCommand.startsWith("MAIL FROM: "))
          CmdMailFrom(strCommand.substring(11));
        else if(strCommand.startsWith("RCPT TO: "))
          CmdRcptTo(strCommand.substring(9));
        else if(strCommand.equals("DATA"))
          CmdData();
        else if(strCommand.equals("QUIT"))
          CmdQuit();
        else
          WriteLine("500 Invalid command");
      }
    }
    catch(java.io.IOException e) {
    }

    try {
      m_Socket.close();
    }
    catch(java.io.IOException e) {
    }
  }

  /**
   * Processes the HELO command
   * @param strHostname Parameter supplied to the HELO command
   * @throws IOException When I/O error occurs
   */
  private void CmdHelo(String strHostname) throws java.io.IOException {
    WriteLine("250 Welcome " + strHostname);
  }

  /**
   * Processes the MAIL FROM: command
   * @param strSenderEmail Parameter supplied to the MAIL FROM: command
   * @throws IOException When I/O error occurs
   */
  private void CmdMailFrom(String strSenderEmail) throws java.io.IOException {
    if(strSenderEmail.startsWith("<") && strSenderEmail.endsWith(">"))
      m_strSenderEmail = strSenderEmail.substring(1, strSenderEmail.length() - 1);
    else
      m_strSenderEmail = strSenderEmail;

    WriteLine("250 " + m_strSenderEmail + " Sender OK");

    m_strRecipientEmail = null;
  }

  /**
   * Processes the RCPT TO: command
   * @param strRecipientEmail Parameter supplied to the RCPT TO: command
   * @throws IOException When I/O error occurs
   */
  private void CmdRcptTo(String strRecipientEmail) throws java.io.IOException {
    if(m_strSenderEmail == null) {
      WriteLine("503 Invalid state");
      return;
    }

    if(strRecipientEmail.startsWith("<") && strRecipientEmail.endsWith(">"))
      m_strRecipientEmail = strRecipientEmail.substring(1, strRecipientEmail.length() - 1);
    else
      m_strRecipientEmail = strRecipientEmail;

    if(AppClass.appClass1.m_MailBoxManager.UserExists(m_strRecipientEmail))
      WriteLine("250 " + m_strRecipientEmail + " Recipient OK");
    else {
      WriteLine("550 " + m_strRecipientEmail + " Recipient not found on this server");
      m_strRecipientEmail = null;
    }
  }

  /**
   * Processes the DATA command
   * @throws IOException When I/O error occurs
   */
  private void CmdData() throws java.io.IOException {
    if(m_strRecipientEmail == null) {
      WriteLine("503 Invalid state");
      return;
    }

    WriteLine("354 Enter mail, end with \".\" on a line by itself");

    ArrayList message = new ArrayList();
    String s;

    while(((s = m_brInput.readLine()) != null) && !s.equals(".")) {
      message.add(s);
    }

    String[] strMessage = new String[message.size()];

    for(int i = 0; i < strMessage.length; i++)
      strMessage[i] = (String)message.get(i);

    AppClass.appClass1.m_MailBoxManager.AddMessage(m_strRecipientEmail, strMessage);

    WriteLine("250 Message delivered");

    m_strRecipientEmail = null;
    m_strSenderEmail = null;
  }

  /**
   * Processes the QUIT command
   * @throws IOException When I/O error occurs
   */
  private void CmdQuit() throws java.io.IOException {
    WriteLine("221 Bye");
    m_bQuit = true;
  }
}