import java.io.*;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.StringTokenizer;

/**   1.5     Java.   
  *
  *       
  * <a href="inetjava.sourceforge.net">  </a>.
  *
  * @version 2.0
  * @author  
  */
public class SimpleFileClient {
    /**       . */
    private static DataInputStream stdIn = new DataInputStream(System.in);

    /**       . */
    private static PrintStream toServer;
    
    /**      . */
    private static DataInputStream fromServer;
    
    public static void main(String[] args) {
        try {
            System.out.println("SimpleFileClient v2.0  " +
                "Copyright (C) 2002 by Neno Galev\n" +
                "Type \"HELP\" for help");
            
            //         .
            if (args.length != 1) {
                System.err.println("\nUsage: java SimpleFileClient <host>");
                System.exit(1);
            }
        
            //    .
            Socket socket = new Socket(args[0], 2001);
            toServer = new PrintStream(socket.getOutputStream());
            fromServer = new DataInputStream(socket.getInputStream());
        
            for ( ; ; ) {
                //     .
                System.out.print("\n>");
                Command cmd = checkUserCommand(stdIn.readLine());
            
                //    ,    
                //  .
                if (cmd == null) {
                    System.err.println("-Invalid command");
                    continue;
                }
            
                //    ,     
                //     .
                switch (cmd.code) {
                case Command.DIR:
                    toServer.println("DIR");
                    printServerLines();
                    break;
            
                case Command.HELP:
                    toServer.println("HELP");
                    printServerLines();
                    break;
            
                case Command.QUIT:
                    toServer.println("QUIT");
                    System.out.println(fromServer.readLine());
                    System.exit(0);
            
                case Command.CD:
                    toServer.println("CD " + cmd.arg);
                    printServerResponse();
                    break;
            
                case Command.GET:
                    get(cmd.arg);
                    break;
            
                default:
                    upload(cmd.arg);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**       .
      *
      * @param userCommand    .
      *
      * @return null,      ,  -
      *      Command .
      */
    private static Command checkUserCommand(String userCommand) {
        StringTokenizer st = new StringTokenizer(userCommand);
        int numTokens = st.countTokens();
        
        //       .
        if (numTokens < 1 || numTokens > 2) return null;
        
        //         .
        String codeStr = st.nextToken().toUpperCase();
        
        //    .
        byte code = codeStr.equals("DIR") || codeStr.equals("LS") ||
            codeStr.equals("LIST") ? Command.DIR :
            codeStr.equals("HELP") ? Command.HELP :
            codeStr.equals("QUIT") ? Command.QUIT :
            codeStr.equals("CD") ? Command.CD :
            codeStr.equals("GET") ? Command.GET :
            codeStr.equals("UPLOAD") ? Command.UPLOAD :
            Command.ILLEGAL;
        if (code == Command.ILLEGAL) return null;

        //        .
        String arg;
        if (code >= Command.DIR && code <= Command.QUIT) {
            if (numTokens != 1) return null;
            arg = null;
        } else {
            if (numTokens != 2) return null;
            arg = st.nextToken();
        }
        
        return new Command(code, arg);
    }
    
    /**           , 
      *  ,     .
      */
    private static void printServerLines() throws IOException {
        String currLine;
        do {
            System.out.println(currLine = fromServer.readLine());
        } while (!currLine.equals("."));
    }
    
    /**      System.out,     
      * System.err,   .
      *
      * @return true,    ,  - false.
      */
    private static boolean printServerResponse() throws IOException {
        String serverResponse = fromServer.readLine();
        boolean positiveResponse = serverResponse.charAt(0) == '+';
        PrintStream out = positiveResponse ? System.out : System.err;
        out.println(serverResponse);
        return positiveResponse;
    }
    
    /**   GET.
      *
      * @param filename   ,      .
      */
    private static void get(String filename)
        throws FileNotFoundException, IOException {
        toServer.println("GET " + filename);
        if (printServerResponse()) {
            //        .
            long fileSize = Long.parseLong(fromServer.readLine());
            System.out.println(fileSize);
            
            //         
            //   .
            BufferedOutputStream file = new BufferedOutputStream(
                new FileOutputStream(filename));
            for ( ; fileSize > 0; fileSize--) {
                int b = fromServer.read();
                System.out.write(b);
                file.write(b);
            }
            file.close();
            System.out.println();
        }
    }
    
    /**   UPLOAD.
      *
      * @param filename   ,      .
      */
    private static void upload(String filename)
        throws FileNotFoundException, IOException {
        //       , 
        //   .
        File file = new File(filename);
        if (!file.exists() || file.isDirectory() || !file.canRead()) {
            System.err.println("-Invalid file");
            return;
        }
        
        long fileSize = file.length();
        toServer.println("SEND " + filename);
        if (printServerResponse()) {
            toServer.println(fileSize);
            BufferedInputStream in = new BufferedInputStream(
                new FileInputStream(filename));
            for ( ; fileSize > 0; fileSize--) {
                toServer.write(in.read());
            }
            in.close();
        }
    }
}

/** ,   ,     . */
class Command {
    /**   . */
    public final byte code;
    
    /**   .      null. */
    public final String arg;
    
    /**    DIR. */
    public final static byte DIR = 0;

    /**    HELP. */
    public final static byte HELP = 1;
    
    /**    QUIT. */
    public final static byte QUIT = 2;
    
    /**    CD. */
    public final static byte CD = 3;
    
    /**    GET. */
    public final static byte GET = 4;
    
    /**    UPLOAD. */
    public final static byte UPLOAD = 5;
    
    /**    . */
    public final static byte ILLEGAL = 6;
    
    /**  Command .
      *
      * @param code   .
      * @param arg   .
      */
    public Command(byte code, String arg) {
        if (code < DIR || code > UPLOAD ||
            code < CD && arg != null) {
            throw new IllegalArgumentException();
        }
        
        this.code = code;
        this.arg = arg;
    }
}
