import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.StringTokenizer;

/**   1.5     Java.   
  *
  *       
  * <a href="inetjava.sourceforge.net">  </a>.
  *
  * @version 2.0
  * @author  
  */
public class SimpleFileServer {
    /**  . */
    public static File rootDir;
    
    /**    . */
    public static String rootPath;
    
    /**       (  ). */
    public static int rootPathLength;
    
    public static void main(String[] args) {
        try {
            System.out.println("SimpleFileServer v2.0  " +
                "Copyright (C) 2002 by Neno Galev");
            
            //       
            //   .
            if (args.length != 1) {
                System.err.println(
                    "Usage: java SimpleFileServer <root directory>");
                System.exit(1);
            }        
            rootDir = new File(args[0]);
            rootPath = rootDir.getCanonicalPath();
            rootPathLength = rootPath.length();
        
            ServerSocket serverSocket = new ServerSocket(2001);
            for ( ; ; ) {
                new SimpleFileClientThread(serverSocket.accept()).start();
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
            System.exit(1);
        }
    }
}

/** ,   . */
class SimpleFileClientThread extends Thread {
    /**      . */
    private DataInputStream fromClient;
    
    /**      . */
    private PrintStream toClient;
    
    /**       . */
    private File currDir = new File(SimpleFileServer.rootPath);
    
    /**         
      *   .
      */
    private String currDirName = File.separator;
    
    /**    .
      *
      * @param socket ,       .
      */
    public SimpleFileClientThread(Socket socket) throws IOException {
        fromClient = new DataInputStream(socket.getInputStream());
        toClient = new PrintStream(socket.getOutputStream());
    }
    
    /**    ,      ,
      *    QUIT.  ,    
      * .
      */
    public void run() {
        try {
            for ( ; ; ) {
                StringTokenizer st =
                    new StringTokenizer(fromClient.readLine());
                String cmd = st.nextToken();
                if (cmd.equals("DIR")) {
                    dir();
                } else if (cmd.equals("HELP")) {
                    toClient.println("+Valid commands are:\n" +
                        "DIR  Lists current directory.\n" +
                        "CD <directory>  Changes current directory.\n" +
                        "GET <filename>  " +
                        "Retrieves given file from current directory.\n" +
                        "SEND <filename>\n<filesize>\n<file contents>  " +
                        "Sends given file to the server.\n" +
                        "HELP  Displays this help.\n" +
                        "QUIT  Closes this session.\n.");
                } else if (cmd.equals("QUIT")) {
                    toClient.println("+Bye");
                    return;
                } else if (cmd.equals("CD")) {
                    cd(st.nextToken());
                } else if (cmd.equals("GET")) {
                    get(st.nextToken());
                } else {
                    send(st.nextToken());
                }
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
            System.exit(1);
        }
    }
    
    /**   DIR. */
    private void dir() throws IOException {
        toClient.println("+Listing directory " + currDirName + "\n");
        
        //       , 
        //         
        //    .
        if (!currDir.getCanonicalPath().equals(SimpleFileServer.rootPath)) {
            toClient.println("..            <DIR>");
        }
        
        File[] fileList = currDir.listFiles();
        int numFiles = 0;
        long totalLength = 0;
        for (int i = 0; i < fileList.length; i++) {
            File currFile = fileList[i];
            String currName = currFile.getName();
            toClient.print(currName);
            
            //     -  12 ,
            for (int j = currName.length(); j <= 12; j++) {
                toClient.print(' ');
            }
            
            //       ,   
            //         
            //     1   .  
            //   "<DIR>".
            if (currFile.isFile()) {
                numFiles++;
                long currLength = currFile.length();
                toClient.println("  " + currLength);
                totalLength += currLength;
            } else {
                toClient.println("  <DIR>");
            }
        }
        toClient.println("\nTotal " + numFiles + " files (" +
            totalLength + " bytes) " + (fileList.length - numFiles) +
            " directories\n.");
    }
        
    /**   CD.
      *
      * @param newDirName   ,    .
      */
    private void cd(String newDirName) throws IOException {
        File newDir = new File(currDir, newDirName);
           
        //         
        //    ,  
        //  .
        String newPath;
        if (!newDir.exists() || !newDir.isDirectory() ||
            (newPath = newDir.getCanonicalPath()).length() <
                SimpleFileServer.rootPathLength ||
            !newPath.substring(0, SimpleFileServer.rootPathLength).
                equals(SimpleFileServer.rootPath)) {
            toClient.println("-Invalid directory");
            return;
        }
            
        currDir = newDir;
                
        //       ,
        //   .
        currDirName = currDir.getCanonicalPath().
            substring(SimpleFileServer.rootPathLength);
        if (currDirName.equals("") ||
            currDirName.charAt(0) != File.separatorChar) {
            currDirName = File.separator + currDirName;
        }
        toClient.println("+Directory changed to " + currDirName);
    }
        
    /**   GET.
      *
      * @param filename   .
      */
    private void get(String filename) throws IOException {
        File file = new File(currDir, filename);
          
        //          ,
        //    .
        if (!file.exists() || !file.isFile() || !file.canRead()) {
            toClient.println("-Invalid file");
            return;
        }
            
        long fileSize = file.length();
        toClient.println("+File size and file contents follow:\n" +
            fileSize);
        BufferedInputStream in = new BufferedInputStream(
            new FileInputStream(file));
        for ( ; fileSize > 0; fileSize--) {
            toClient.write(in.read());
        }
        in.close();
    }
        
    /**   SEND.
      *
      * @param filename   .
      */
    private void send(String filename)
        throws FileNotFoundException, IOException {
        File file = new File(currDir, filename);
            
        //         ,
        //    .
        if (file.exists()) {
            toClient.println("-Cannot store file in current directory");
            return;
        }
            
        toClient.println("+File " + filename + " stored");
        BufferedOutputStream out = new BufferedOutputStream(
            new FileOutputStream(file));
        long fileSize = Long.parseLong(fromClient.readLine());
        for ( ; fileSize > 0; fileSize--) {
            out.write(fromClient.read());
        }
        out.close();
    }
}
