package fileserver;

/*
 * Created by IntelliJ IDEA.
 * User: peter
 * Date: Apr 17, 2002
 * Time: 3:01:01 PM
 */

import utils.*;

import java.io.*;
import java.net.*;
import java.util.Properties;

/**
 * The main server class.
 */
public class FileServer {
	public static final int DEFAULT_PORT = 2002;
	private static CommandParser parser;
	private static String home;

	/**
	 * This method opens a ServerSocket, which waits for connections and starts a new SingleFileServer every
	 * time a connection is established.
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			int port = DEFAULT_PORT;
			if (args.length > 0) {
				try {
					port = Integer.parseInt(args[0]);
					if (port <= 1024) {
						port = DEFAULT_PORT;
						System.out.println("Illegal port number: must be over 1024");
					}
				} catch (NumberFormatException e) {
					System.out.println("Illegal port number");
				}
			}

			parser = new CommandParser();
			parser.init("servercommands.properties");

			Properties props = new Properties();
			props.load(new FileInputStream("server.properties"));
			home = props.getProperty("home_directory");
			if (home == null) {
				throw new IOException("User home must be specified in the properties file: home_directory=<path>");
			}
			File homeFile = new File(home);
			if (!homeFile.exists() || !homeFile.isDirectory()) {
				throw new IOException("User home must be a directory: home_directory=<legal_directory_path>");
			}
			home = homeFile.getCanonicalPath();
			if (home.endsWith(File.separator) || home.endsWith("/"))
				home = home.substring(0, home.length() - 1);
			home = Util.replace(home, File.separator, "/");

			ServerSocket s = new ServerSocket(port);
			System.out.println("Server started on port " + port);
			try {
				while (true) {
					// Blocks until a connection occurs:
					Socket socket = s.accept();
					try {
						System.out.println("Client connected on " + socket);
						new SingleFileServer(socket, parser, home);
					} catch (IOException e) {
						// If it fails, close the socket,
						// otherwise the thread will close it:
						try {
							if (socket != null)
								socket.close();
						} catch (IOException ex) {
							System.out.println("Could not close socket");
							ex.printStackTrace();
						}
					}
				}
			} finally {
				try {
					if (s != null)
						s.close();
				} catch (IOException e) {
					System.out.println("Could not close server socket");
					e.printStackTrace();
				}
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
}

