package utils;

/*
 * Created by IntelliJ IDEA.
 * User: peter
 * Date: Apr 17, 2002
 * Time: 4:50:29 PM
 */

import java.util.Properties;
import java.util.*;
import java.io.*;

/**
 * This class is used by both the client and the server to read the command properties file.
 * It has methods to check if a command is legal, to check how many arguments a certain command expects
 * and to get the name of the method which has to be executed for a certain command.
 */
public class CommandParser {
	Map cmdParameters = new HashMap();
	Map cmdMethods = new HashMap();
	boolean initialized = false;

	public void init(String filename) throws IOException {
		Properties props = new Properties();
		props.load(new FileInputStream(filename));
		for (Enumeration enum = props.propertyNames(); enum.hasMoreElements();) {
			String key = (String)(enum.nextElement());
			String value = props.getProperty(key);
			addCommand(key, value);
		}
		initialized = true;
	}

	private void addCommand(String key, String value) {
		StringTokenizer valueTokens = new StringTokenizer(value, ", \t");
		Integer numArgs = new Integer(valueTokens.nextToken());
		String methodName = valueTokens.nextToken();
		StringTokenizer keyTokens = new StringTokenizer(key, ", \t");
		while (keyTokens.hasMoreTokens()) {
			String cmd = keyTokens.nextToken().toLowerCase().trim();
			cmdParameters.put(cmd, numArgs);
			cmdMethods.put(cmd, methodName);
		}
	}

	public int getNumberOfArguments(String cmd) throws IllegalArgumentException, IllegalStateException {
		if (!initialized) throw new IllegalStateException("Configurator not initialized");
		if (!cmdParameters.containsKey(cmd.toLowerCase().trim()))
			throw new IllegalArgumentException("No such command");
		return ((Integer)cmdParameters.get(cmd.toLowerCase().trim())).intValue();
	}

	public String getMethodName(String cmd) throws IllegalArgumentException, IllegalStateException {
		if (!initialized) throw new IllegalStateException("Configurator not initialized");
		if (!cmdMethods.containsKey(cmd.toLowerCase().trim()))
			throw new IllegalArgumentException("No such command");
		return (String)cmdMethods.get(cmd.toLowerCase().trim());
	}

	public boolean isLegalCommandName(String cmd) {
		return cmdMethods.containsKey(cmd.toLowerCase().trim());
	}

	public boolean isLegalCommand(String command) {
		CommandTokenizer tokens = new CommandTokenizer(command);
		String cmd = tokens.nextToken().toLowerCase().trim();
		return (isLegalCommandName(cmd) && tokens.countTokens()==getNumberOfArguments(cmd));
	}
}
