package simpleproxyserver;

/**
 * Title:        Simple Proxy Server
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author Plamen Pironkov
 * @version 1.0
 */
import java.net.*;
import java.io.*;

/** This class is a thread which listen on the specified port for
 *  client requests. When such a request come it makes new thread
 *  (of class ProxyThread) which do a real proxy work. Thus is
 *  guarantee the multi user support.*/
public class ListenThread extends Thread
{
  /** Shows if the server is ready for receiving requests. Its default
   *  value is true.*/
  public boolean isNotReady = true;
  /** Shows if the port is not used alreday. Its default
   *  value is true.*/
  public boolean isServerPortOK = true;
  /** This boolean shows if the server should show in the console
   *  log. If this variable is false no log is shown in the console.*/
  boolean log;
  /** The port where the server is listening.*/
  int port;

  /** The server socket which listens for requests.*/
  private ServerSocket server;

  /** The constructor of the class. It initializes the port and the log.
   *  @param port The port in which is listening.
   *  @param log shows if there has to be log in the console.*/
  public ListenThread(int port, boolean log)
  {
    this.port = port;
    this.log = log;
  }

  /** Close the server socket.*/
  public synchronized void closeServerSocket()
  {
    if (server != null)
    {
      try
      {
        Socket socket;

        server.close();
        server = null;
        socket = new Socket(InetAddress.getLocalHost(), port);
        socket.close();
      }
      catch (Exception _){}
    }
    /** @todo stop all user's threads*/
  }

  /**The overridden run method. This method do the whole work. It listens
   * while a request come. Then a new thread is done(object of class ProxyThread).
   * The new thread get the client socket as parameter. This new thread
   * do the main proxy work.*/
  public void run()
  {
    server = null;

    try
    {
      server = new ServerSocket(port);
      isNotReady = false;

      while (true)
      {
        Socket client = server.accept();

        ProxyThread theProxyThread = new ProxyThread(client, log);
        theProxyThread.start();
      }
    }
    catch(IOException e)
    {
      isServerPortOK = false;
      isNotReady = false;
    }
    closeServerSocket();
  }
}