package simpleproxyserver;

/**
 * Title:        Simple Proxy Server
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author Plamen Pironkov
 * @version 1.0
 */
import java.util.*;
import java.net.*;
import java.io.*;

/** The main class which processes the client request.*/
public class ProxyThread extends Thread
{
  /** The used protocol. In our case this is HTTP/1.0.*/
  private static final String Protocol = "HTTP/1.0";
  /** Shows if there has to be log.*/
  boolean log;
  /** The client socket. This is the socket of the cliet. From it we
   *  can read the client request and send the server response.*/
  private Socket client;
  /** This client socket is used to make connection with the real
   *  HTTP server. Using it we can forward the client request to
   *  the real HTTP server and to read the response.*/
  private Socket HTTPSocketClient;

  /** The host of the requested data(site).*/
  private String Host = null;
  /** The port of the real HTTP server.*/
  private int Port = 80;
  /** The address of the requested data(site).*/
  private String Address = null;
  /** The whole user's request. The request is a string vector. Each its
   *  elemet is a line from the request.*/
  private Vector Request = null;

  /** The constructor of the class. It initialize the client socket
   *  and the log.
   *  @param client the socket wich we use to make conection with the user
   *  @log Booloean wich shows if there has to have log in the console.*/
  public ProxyThread(Socket client, boolean log)
  {
    this.client = client;
    this.log = log;

    Request = new Vector();
  }

  /** The run method. This method first parse the client request.
   *  This is by the method CreateNewHTTPRequest. After calling of
   *  this function all needed data is initialized(address, host, etc.).
   *  An analysis is done if the requeste host is not forbidden.
   *  If yes the relevant response is send to the user. Otherwise a
   *  new connection with the HTTP server is done and the client
   *  request is forwarded(with some changes). The response is then
   *  redirected to the client.*/
  public void run()
  {
    OutputStream os = null;
    BufferedWriter reswriter = null;;
    InputStream newis = null;
    OutputStream newos = null;
    BufferedWriter webwriter = null;

    try
    {
      CreateNewHTTPRequest();

      os = client.getOutputStream();
      reswriter = new BufferedWriter(new OutputStreamWriter(os));

      if(Address == null || Address.equals(""))
      {
        //error
        SendResponseNotFoud(reswriter);
      }
      //get the host and the port
      URL temp = new URL(Address);
      if(Host == null)
        Host = temp.getHost();
      if(temp.getPort() != -1)
        Port = temp.getPort();
      int t = Host.indexOf(":");
      if(t != -1)
        Host = Host.substring(0, t);

      //if the host is .bg
      if(Host.indexOf(".bg") != -1)
      {
        SendResponseForbidden(reswriter);

        reswriter.close();
        client.close();
        return;
      }

      //make the connection to the Web server
      HTTPSocketClient = new Socket(Host, Port);

      //get the i/o streams
      newis = HTTPSocketClient.getInputStream();
      newos = HTTPSocketClient.getOutputStream();
      webwriter = new BufferedWriter(new OutputStreamWriter(newos));

      //write the request
      webwriter.write("GET " + temp.getPath() + " HTTP/1.0\n");
      for(int i = 1; i < Request.size(); i++)
      {
        webwriter.write(((String)Request.elementAt(i)) + "\n");
      }
      webwriter.write("\n");
      webwriter.flush();

      //read the response and send it to the client.
      System.out.println("___________________________________________________________");
      System.out.println("THE RESPONSE IS:");
      System.out.println();

      byte[] buf = new byte[8192];
      int len = newis.read(buf);
      while(len > 0)
      {
        String s = new String(buf, 0, len);
        if(log)
          System.out.print(s);
        reswriter.write(s);
        len = newis.read(buf);
      }
      System.out.println("END OF THE RESPONSE:");
      System.out.println("___________________________________________________________");

      //close the streams and the sockets.
      reswriter.flush();
      reswriter.close();
      newis.close();
      webwriter.close();

      HTTPSocketClient.close();
      client.close();
    }
    catch(UnknownHostException excp)
    {
      SendResponseNotFoud(reswriter);

      try
      {
        reswriter.close();
        newis.close();
        webwriter.close();

        HTTPSocketClient.close();
        client.close();
      }
      catch(Exception e1){}
    }
    catch(MalformedURLException exc)
    {
      SendResponseNotFoud(reswriter);

      try
      {
        reswriter.close();
        newis.close();
        webwriter.close();

        HTTPSocketClient.close();
        client.close();
      }
      catch(Exception e2){}
    }
    catch(IOException ex)
    {
      try
      {
        reswriter.close();
        newis.close();
        webwriter.close();

        HTTPSocketClient.close();
        client.close();
      }
      catch(Exception e3){}
    }
  }
  /** This function parse the HTTP request from the client. It takes
   *  the address and host of the requested data(site), the host of the
   *  client. it also fill the vector Request with all data.*/
  private void CreateNewHTTPRequest()
  {
    try
    {
      if(log)
      {
        System.out.println("___________________________________________________________");
        System.out.println("NEW CLIENT REQUEST.");
        System.out.println("FROM: " + client.getInetAddress().toString());
        System.out.println();
        System.out.println("REQUEST DATA:");
      }
      InputStream clientIS = client.getInputStream();

      BufferedReader theReader = new BufferedReader(new InputStreamReader(clientIS));
      String line = theReader.readLine();
      while((line != null) && (!line.equals("")))
      {
        int HostIndex;
        int GetIndex = line.indexOf("GET");
        if(GetIndex != -1)
        {
          int ProtocolIndex = line.indexOf("HTTP");
          if(ProtocolIndex != -1)
          {
            Address = line.substring((GetIndex + 4), (ProtocolIndex - 1));
            String newLine = line.substring(GetIndex, ProtocolIndex) + Protocol;
            Request.addElement(newLine);
          }
        }
        else if((HostIndex = line.indexOf("Host: ")) != -1)//this is not the GET line
        {
          Host = line.substring(HostIndex + 6);
          Request.addElement(line);
        }
        else
          Request.addElement(line);

        if(log)
          System.out.println(line);

        line = theReader.readLine();
      }
      if(log)
      {
        System.out.println("END OF REQUEST");
        System.out.println("___________________________________________________________");
      }
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** Just following the HTTP protocol this method generate an
   *  response "Not Found" and send it to the user.*/
  private void SendResponseNotFoud(BufferedWriter reswriter)
  {
    try
    {
      reswriter.write("HTTP/1.0 404 Not Found");
      reswriter.newLine();
      reswriter.write("MIME-Version: 1.0");
      reswriter.newLine();
      reswriter.write("Content-type: text/html");
      reswriter.newLine();
      reswriter.newLine();
      reswriter.write("<html>");
      reswriter.newLine();
      reswriter.write("Requested document not found!");
      reswriter.newLine();
      reswriter.write("</html>");
    }
    catch(Exception e){}
  }

  /** Just following the HTTP protocol this method generate an
   *  response "Forbidden" and send it to the user.*/
  private void SendResponseForbidden(BufferedWriter reswriter)
  {
    try
    {
      reswriter.write("HTTP/1.1 403 Forbidden");
      reswriter.newLine();
      reswriter.write("MIME-Version: 1.0");
      reswriter.newLine();
      reswriter.write("Content-type: text/html");
      reswriter.newLine();
      reswriter.newLine();
      reswriter.write("<html>");
      reswriter.newLine();
      reswriter.write("<p align=\"center\">");
      reswriter.newLine();
      reswriter.write("Access to .BG domain is restricted.");
      reswriter.newLine();
      reswriter.write("Contact your system administrator for more details.");
      reswriter.newLine();
      reswriter.write("</p>");
      reswriter.newLine();
      reswriter.write("</html>");
    }
    catch (Exception ex){}
  }
}