package simpleproxyserver;

/**
 * Title:        Simple proxy server
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author Plamen Pironkov
 * @version 1.0
 */

import java.util.*;
import java.net.*;
import java.io.File;

/** This is the main class for the Simple Proxy Server. It starts the
 *  application. SimpleProxyServerBoot start the server, which is
 *  responsible for managing the client requests(object from type
 *  ListenThread), in separate thread. this is done because we don't
 *  want the proxy server to take a lot of resources.  */
public class SimpleProxyServerBoot
{
  /** The thread which has to be started and which managing client
   *  requests.*/
  private ListenThread theListenThread;
  /** The port of the server. Clients can make a connection with the
   *  server on this specified port. There can be a  user interface
   *  wich can change the port(this is only an idea).*/
  int port;

  /** The constructor of the class. It initialize the port and start the
   *  listen thread. While starting the ListenThread a sleep() method is
   *  called in order to make the server not ocupating all processor time.
   *  @param port the port on which the server should be listening.*/
  public SimpleProxyServerBoot(int port)
  {
    this.port = port;

    theListenThread = new ListenThread(port, true);
    theListenThread.start();
    do
    {
      try
      {
        Thread.sleep(50);
      }
      catch (InterruptedException _) {}
    }
    while (theListenThread.isNotReady);

    if (!theListenThread.isServerPortOK)
    {
      System.out.println("Error during starting.");
      System.exit(0);
    }

    System.out.println("Proxy server started. Stop with <CTRL-C>.");
  }

  /** The main() method of the application. It just call the constructor
   *  of the class wit parameter 3128 - the port on which the server
   *  will listen.*/
  public static void main(String[] args)
  {
    new SimpleProxyServerBoot(3128);
  }
}