package inetjava_project_1_3;

/**
 * <p>Title: Simple web proxy</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Stoyan Iordanov
 * @version 1.0
 */

import java.net.*;
import java.io.*;
import java.util.*;

public class ProxyInstance extends Thread {
  Socket m_Socket;
  Socket m_WebServerSocket;
  ArrayList m_Request;
  String m_strHostAddress;
  String m_strAddressPath;
  int m_iPort;

  private class ForbiddenException extends Exception {
    private String message;

    public ForbiddenException(String s) {
      message = s;
    }

    public String getMessage() {
      return message;
    }
  }

  private class NotFoundException extends Exception {
    private String message;

    public NotFoundException(String s) {
      message = s;
    }

    public String getMessage() {
      return message;
    }
  }

  public ProxyInstance(Socket s) {
    m_Socket = s;
    m_Request = new ArrayList();
  }

  public void run() {
    try {
      GetRequest();
      ProcessRequest();
      DoForwarding();
    }
    catch(NotFoundException e) {
      ReturnError("404 Not Found", e.getMessage());
    }
    catch(ForbiddenException e) {
      ReturnError("403 Forbidden", e.getMessage());
    }
    catch(Exception e) {
    }

    try {
      m_Socket.close();
    }
    catch(IOException e) {
    }

    m_Socket = null;
  }

  /**
   * Returns an error to the browser
   * @param strErrCode Error code (for example "404 Not Found")
   * @param strErrDescription Error description
   */
  private void ReturnError(String strErrCode, String strErrDescription) {
    try {
      BufferedWriter bwOutput =
        new BufferedWriter(new OutputStreamWriter(m_Socket.getOutputStream()));

      bwOutput.write("HTTP/1.0 " + strErrCode);
      bwOutput.newLine();

      bwOutput.write("MIME-Version: 1.0");
      bwOutput.newLine();

      bwOutput.write("Content-type: text/html");
      bwOutput.newLine();

      bwOutput.newLine();

      bwOutput.write("<html>");
      bwOutput.newLine();

      bwOutput.write("<p align=\"center\">");
      bwOutput.newLine();

      bwOutput.write(strErrDescription);
      bwOutput.newLine();

      bwOutput.write("</p>");
      bwOutput.newLine();

      bwOutput.write("</html>");
      bwOutput.newLine();

      bwOutput.flush();
    }
    catch(Exception e) {
    }
  }

  /**
   * Retrieves the HTTP request from the browser
   * @throws IOException
   */
  private void GetRequest() throws java.io.IOException {
    BufferedReader brInput =
      new BufferedReader(new InputStreamReader(m_Socket.getInputStream()));

    String str;

    while(!(str = brInput.readLine()).equals("")) {
      m_Request.add(str);
    }
  }

  /**
   * Retrieves and validates web server's address and port (which includes
   * filtering of .bg sites), modifies the request (changes HTTP 1.1 to 1.0,
   * etc.), and opens a connection to the web server.
   */
  private void ProcessRequest() throws NotFoundException, ForbiddenException {
    String s;

    // Get the first line of the request - it should be a GET command
    try {
      s = (String)m_Request.get(0);
    }
    catch(IndexOutOfBoundsException e) {
      throw new NotFoundException("Empty request!");
    }

    if(!s.startsWith("GET ")) {
      throw new NotFoundException("GET expected!");
    }

    // Extract the requested address
    int iAddressStart, iAddressEnd;

    iAddressStart = s.indexOf("://");
    if(iAddressStart == -1)
      iAddressStart = 4; // GET address
    else
      iAddressStart += 3; // skip the "://" part

    iAddressEnd = s.indexOf(' ', iAddressStart);
    if(iAddressEnd == -1)
      iAddressEnd = s.length();

    String strAddress = s.substring(iAddressStart, iAddressEnd);

    // Extract the host and path addresses
    int iPathStart = strAddress.indexOf('/');
    if(iPathStart == -1) {
      // No slash -> everything is a host address
      m_strHostAddress = strAddress;
      m_strAddressPath = "/";
    }
    else if(iPathStart == 0) {
      // Slash at the beginning -> everything is a path address
      m_strAddressPath = strAddress;

      // We extract the host address from the "Host: " line of the request
      int iRequestSize = m_Request.size();
      int i;
      for(i = 0; i < iRequestSize; i++) {
        String strRequestLine = (String)m_Request.get(i);

        if(strRequestLine.startsWith("Host: ")) {
          m_strHostAddress = strRequestLine.substring(6);
          break;
        }
      }

      if(i == iRequestSize) {
        // Host not found
        throw new NotFoundException("Host not found!!!");
      }
    }
    else {
      // Slash separating the host and path addresses
      m_strHostAddress = strAddress.substring(0, iPathStart);
      m_strAddressPath = strAddress.substring(iPathStart);
    }

    // Extract the port (if we have one), and remove it from the host address
    int iPortStart = m_strHostAddress.indexOf(':');
    if(iPortStart == -1)
      m_iPort = 80; // HTTP
    else {
      try {
        m_iPort = Integer.parseInt(m_strHostAddress.substring(iPortStart + 1));
      }
      catch(Exception e) {
        m_iPort = 80; // HTTP
      }

      m_strHostAddress = m_strHostAddress.substring(0, iPortStart);
    }

    // Replace the GET line of the request with the appropriate one
    // (relative address, and HTTP/1.0)
    m_Request.set(0, "GET " + m_strAddressPath + " HTTP/1.0");

    // We check if the address is in the .bg zone
    if((m_strHostAddress.length() > 3) &&
       (m_strHostAddress.substring(m_strHostAddress.length() - 3).equalsIgnoreCase(".bg")))
      throw new ForbiddenException("Addresses in the .bg zone are forbidden!!!");

    // Now we know everything we need to open the connection
    try {
      m_WebServerSocket = new Socket(m_strHostAddress, m_iPort);
    }
    catch(Exception e) {
      throw new NotFoundException("Host not found!");
    }
  }

  /**
   * Sends the request to the web server and performs data forwarding
   * from the server back to the web browser.
   * @throws IOException
   */
  private void DoForwarding() throws java.io.IOException {
    // Connect to the web server
    OutputStream osOutput = m_Socket.getOutputStream();
    InputStream isWebServerInput = m_WebServerSocket.getInputStream();
    BufferedWriter bwWebServerOutput =
      new BufferedWriter(new OutputStreamWriter(m_WebServerSocket.getOutputStream()));

    // Send the request to the web server
    int iRequestSize = m_Request.size();
    for(int i = 0; i < iRequestSize; i++) {
      bwWebServerOutput.write((String)m_Request.get(i));
      bwWebServerOutput.newLine();
    }
    bwWebServerOutput.newLine();
    bwWebServerOutput.flush();

    // Perform webserver-to-client forwarding
    int b;
    while((b = isWebServerInput.read()) != -1) {
      osOutput.write(b);
    }

    osOutput.flush();
  }
}