import java.io.*;
import java.net.*;
import java.lang.*;

class HTTPClientThread extends Thread {
	static File wwwRoot;
	static final String GETString = "GET ";
	static final String HTTPString = "HTTP";
	static final String ProtocolString = "HTTP/1.1";
	static final String ContentTypeString = "Content-Type: ";
	static final String[] HTTPResponses = {
		"200 OK",
		"400 Bad Request",
		"404 Object not found"
	};
	static final String[] HTTPErrorHTML = {
		"",
		"<HTML><H1>400 Bad Request</H1><H2>Your browser has issued an invalid request</H2></HTML>",
		"<HTML><H1>404 File not found</H1><H2>The file you requested could not be found on the server</H2></HTML>"
	};
	
	static final int HTTP_RESPONSE_OK = 0;
	static final int HTTP_RESPONSE_BADREQUEST = 1;
	static final int HTTP_RESPONSE_NOTFOUND = 2;
	static final String defaultMimeType = "text/plain";
	static final String HTMLMimeType = "text/html";
	static final String[][] MimeTypes = {
		{"gif","image/gif"},
		{"jpg","image/jpeg"},
		{"jpeg","image/jpeg"},
		{"html","text/html"},
		{"css","text/css"},
		{"js","text/javascript"},
		{"txt","text/plain"}
	};
	static final String HTTPHeader = "<HTML><HEAD><TITLE>Listing</TITLE></HEAD><BODY>";
	static final String HTTPFooter = "</BODY></HTML>";
	
	private Socket socket;
	private BufferedReader input;
	private PrintWriter output;
	private BufferedOutputStream outbin;
	private File file;
	private BufferedInputStream fins;
	
	private String getMimeType() throws IOException {
		String extension = file.getName().substring(file.getName().lastIndexOf('.')+1).toLowerCase();
		for(int i=0;i<MimeTypes.length;i++) {
			if (extension.equals(MimeTypes[i][0])) return MimeTypes[i][1];
		}
		return defaultMimeType;
	}
	
	private String relativeURL(File file) {
		try {
		return file.toURL().toString().substring(wwwRoot.toURL().toString().length()-1);
		} catch (MalformedURLException e) {return "";}
	}
		
	private int processHTTPQuery(String[] query) {
		if (query==null || query[0]==null)
		  return HTTP_RESPONSE_BADREQUEST;
		query[0].trim();
		if ( query[0].toUpperCase().startsWith(GETString) &&
		     query[0].substring(query[0].lastIndexOf(' ')+1).toUpperCase().startsWith(HTTPString)) {
				String resource = new String(query[0].substring(GETString.length())).trim();
				resource = resource.substring(0,resource.indexOf(' '));
				resource = URLDecoder.decode(resource);		
				System.out.println("Opening file "+resource);
				file = new File(wwwRoot,resource);
				if (!file.exists()) return HTTP_RESPONSE_NOTFOUND;
				if (file.isFile()) try {
					fins = new BufferedInputStream(new FileInputStream(file));
					if (fins==null) return HTTP_RESPONSE_NOTFOUND;
				} catch (FileNotFoundException e) {
					return HTTP_RESPONSE_NOTFOUND;
				}
			} else return HTTP_RESPONSE_BADREQUEST;
		return HTTP_RESPONSE_OK;
	}			

	public HTTPClientThread(Socket s) throws IOException {
        // throw the exception to the calling class
		socket = s;
		input = new BufferedReader(
				new InputStreamReader(socket.getInputStream()));
		output = new PrintWriter(new BufferedWriter(
				new OutputStreamWriter(socket.getOutputStream())));
		outbin = new BufferedOutputStream(socket.getOutputStream());
		start();
	}
	
	public void run() {
		try {
				String[] stringArray = new String[10];
				String str;
				int i = 0;
				while ((str = input.readLine())!=null &&
						!str.equals("") && i<10)
					stringArray[i++] = str;
				int HTTPCode = processHTTPQuery(stringArray);
				output.println(ProtocolString+" "+HTTPResponses[HTTPCode]);
				output.flush();
				if (HTTPCode==HTTP_RESPONSE_OK) {
					if (file.isDirectory()) {
						output.println(ContentTypeString+HTMLMimeType);
						output.println();
						output.println(HTTPHeader);
						output.println("<H1>Listing of "+relativeURL(file)+"</H1><TABLE>");
						File[] dirList = file.listFiles();
						for(i=0;i<dirList.length;i++) {
							String fileDesc = dirList[i].isDirectory() ? "&lt;DIR&gt;" : ""+dirList[i].length();
							output.println("<TR><TD>&nbsp;&nbsp;</TD><TD><A href=\""+relativeURL(dirList[i])+"\">"+dirList[i].getName()+"</A>&nbsp;&nbsp;&nbsp;&nbsp;</TD><TD align=right>"+fileDesc+"</TD></TR>");
						}
						output.println("</TABLE>"+HTTPFooter);
						output.flush();
						// print listing;
					} else {
						output.println(ContentTypeString+getMimeType());
							output.println();
						output.flush();
						int a;
						if (fins==null) System.out.println("ERROR!!!");
						while ((a = fins.read()) != -1 )
					  		outbin.write(a);
						outbin.flush();
					}
				} else {
					output.println(ContentTypeString+HTMLMimeType);
					output.println();
					output.println(HTTPErrorHTML[HTTPCode]);
					output.flush();
				}	
		} catch (IOException e) {}
		finally {
			try {
				socket.close();
			} catch (Exception e) {}
		}
	}
}


class HTTPServer {
	static final int port = 80;
	
	public static void main(String[] args)
		throws IOException
	{
	    if (args.length > 0 && args[0]!=null) {
		HTTPClientThread.wwwRoot = new File(args[0]);
		if (!HTTPClientThread.wwwRoot.isAbsolute()) {
			System.out.println("Absolute path expected!");
			System.exit(1);
		}
	    }
	    else HTTPClientThread.wwwRoot = new File(new File("").getAbsolutePath());
	    ServerSocket socket = new ServerSocket(port);
	    System.out.println("Server listening on port "+port);
	    try {	    	
	    	while (true)
	    	{
		  	Socket s = socket.accept();
	  		try {
	            	new HTTPClientThread(s);
	    		} catch (IOException e) {
			        s.close();
	    		}
	    	}	    	
	    } finally {
	    	socket.close();
	    }
	}
}
