/*
 * Simple Proxy Server.
 * author: Vasil Dimov, vd@etrade.bg
 */

import java.io.*;
import java.util.*;
import java.net.*;

// class for error messages and a method for getting them
class Err {
  private static final String[] errz = {
    "HTTP/1.0 200 OK\n\n",

    "HTTP/1.0 400 Bad Request\n" +
    "Content-type: text/html\n\n" +                                                                 
    "<html><body>\n" +
    "Bad request.\n" +
    "</body></html>\n",

    "HTTP/1.0 403 Forbidden\n" +
    "Content-type: text/html\n\n" +
    "<html><body>\n" +
    "Currently you do not have access to the object requested.<br>\n" +
    "Please do not contact your system administrator.\n" +
    "</body></html>\n",

    "HTTP/1.0 404 Not Found\n" +
    "Content-type: text/html\n\n" +
    "<html><body>\n" +
    "The requested document was not found.\n" +
    "</body></html>\n",

    "HTTP/1.0 503 Service Unavailable\n" +
    "Content-type: text/html\n\n" +                                                                 
    "<html><body>\n" +
    "The requested document is temporary unavailable.\n" +
    "</body></html>\n"};

  public static String strerr(int errnum) {
    if (errnum < 0 || errz.length - 1 < errnum) {
      return errz[1];
    }
    return errz[errnum];
  }
}

// Client thread class, one instance for each client
class ClntThrd extends Thread {

  Socket cpsck, pssck;  // client<->proxy and proxy<->server sockets

  BufferedReader cp_strm;  // client->proxy stream
  BufferedWriter ps_strm;  // proxy->server stream
  InputStream sp_strm;  // proxy<-server stream
  OutputStream pc_strm;  // client<-proxy stream

  final static int BUFSIZE = 4096;  // sp_strm and pc_strm r/w bufsize

  byte[] spcbuf = new byte[BUFSIZE];  // server->proxy->client buffer

  String ln, cstr;  // line, connection string

  Err errh;  // error handler

  // constructor
  public ClntThrd(Socket s) {
    try {

      cpsck = s;

      cp_strm = new BufferedReader(new InputStreamReader(cpsck.getInputStream()));
      pc_strm = cpsck.getOutputStream();

      cstr = cpsck.getInetAddress().getHostAddress() + ":" + cpsck.getPort() + " -> " +
             cpsck.getLocalAddress().getHostAddress() + ":" + cpsck.getLocalPort();

      errh = new Err();

    } catch (IOException e) {
      System.err.println("IOerr: " + e.getMessage());
    }
  }

  // called before return if some err is found
  private void err(int errnum) {
    try {

      pc_strm.write(errh.strerr(errnum).getBytes());
      pc_strm.flush();

      cpsck.close();
    } catch (IOException e) {
      System.err.println("IOerr: " + e.getMessage());
    }

    System.out.println(cstr + " : closed (" + errnum + ")\n");
  }

  public void run() {
    String url, host, bufstr;
    StringBuffer fln;  // first line, needs to be changed : HTTP/1.1 -> HTTP/1.0

    Vector lnz = new Vector(16);  // linez vector, holds the whole query

    int port = 80, i, ii, lnzcnt, len, f;

    InetAddress host_addr;

    try {

      System.out.println(cstr + " : accepted");

      // read the query from client and put it in lnz
      for (ln = cp_strm.readLine();
           ln != null && !ln.equals("");
           ln = cp_strm.readLine()) {
        lnz.add(ln);
      }

      lnzcnt = lnz.size();

      // query must contain at least one line
      if (lnzcnt == 0) {
        err(1);
        return;
      }

      // print qry in server conzole
      System.out.print(cstr + " : query :\n");
      for (i = 0; i < lnzcnt; i++) {
        System.out.print(lnz.get(i).toString() + "\n");
      }

      // check if first line is somethig like "^GET\ .+\ HTTP/1.[01]$"
      // unfortunately Java has no idea what re is
      bufstr = lnz.get(0).toString();
      if (bufstr.length() < 14 ||
          !bufstr.substring(0, 4).equals("GET ") ||
          !(bufstr.substring(bufstr.lastIndexOf(' ') + 1).equals("HTTP/1.0") ||
            bufstr.substring(bufstr.lastIndexOf(' ') + 1).equals("HTTP/1.1"))) {
        err(1);  // bad request
        return;
      }

      url = bufstr.substring(4, bufstr.lastIndexOf(' '));

      len = url.length();
      // check if url begins with http:// and get host from it in that case
      // host is "^http://([^:/]*).*$" from url
      if (len > 7 && url.substring(0, 7).equalsIgnoreCase("http://")) {
        for (i = 7; i < len && url.charAt(i) != ':' && url.charAt(i) != '/'; i++);
        host = url.substring(7, i);
        if (i < len && url.charAt(i) == ':') {  // get port if any
          for (ii = i + 1; ii < len && url.charAt(ii) != '/'; ii++);
          try {
            port = Integer.parseInt(url.substring(i + 1, ii));
          } catch (NumberFormatException e) {
            err(1);  // bad request, the thing following ':' is not a number
            return;
          }
        }
      } else {  // err(1) should be called at this point. nakov rlz
        host = "fucked java compiler tries to be intelligent";
        if (lnzcnt < 2) {
          err(1);
          return;
        }
        f = 0;
        for (i = 1; i < lnzcnt && f == 0; i++) {
          bufstr = lnz.get(i).toString();
          len = bufstr.length();
          if (len > 6 && bufstr.substring(0, 6).equalsIgnoreCase("Host: ")) {
            for (i = 6; i < len && bufstr.charAt(i) != ':'; i++);
            host = bufstr.substring(6, i);
            if (i < len) {
              try {
                port = Integer.parseInt(bufstr.substring(i + 1));
              } catch (NumberFormatException e) {
                err(1);
                return;
              }
            }
            f = 1;
          }
        }
        if (f == 0) {
          err(1);
          return;
        }
      }

      // reject access to BG tld
      len = host.length();
      if (host.substring(len - 3).equalsIgnoreCase(".bg") ||
          host.substring(len - 4).equalsIgnoreCase(".bg.")) {
        err(2);  // forbidden
        return;
      }

      // get IP addr of the web server requested
      try {
        host_addr = InetAddress.getByName(host);
      } catch (Exception e) {
        err(3);  // not found
        return;
      }

      // avoid loops
      // NOTE: java does not have method for retrieving all addresses,
      // to which ServerSocket is bound, so we catch the second loop
      if (cpsck.getLocalPort() == port && cpsck.getLocalAddress().equals(host_addr)) {
        System.err.println("rejecting loop from client");
        err(2);  // forbidden
        return;
      }

      // connect to the requested server
      try {
        pssck = new Socket(host_addr, port);
      } catch (Exception e) {
        err(3);  // err(4) should be called, nakov rlz
        return;
      }

      sp_strm = pssck.getInputStream();
      ps_strm = new BufferedWriter(new OutputStreamWriter(pssck.getOutputStream()));

      // change last char in first line to 0
      fln = new StringBuffer(lnz.get(0).toString());
      fln.setCharAt(fln.length() - 1, '0');

      try {
        // send first line to server separately
        ps_strm.write(fln.toString(), 0, fln.toString().length());
        ps_strm.newLine();
        // send the rest of the query
        for (i = 1; i < lnzcnt; i++) {
          ps_strm.write(lnz.get(i).toString(), 0, lnz.get(i).toString().length());
          ps_strm.newLine();
        }
        // put one extra header to indicate who is the client behind the proxy
        bufstr = "X-Forwarded-For: " + cpsck.getInetAddress().getHostAddress();
        ps_strm.write(bufstr, 0, bufstr.length());
        ps_strm.newLine();
        // end of query
        ps_strm.newLine();
        ps_strm.flush();

        // read from server and send to client 1:1
        while ((ii = sp_strm.read(spcbuf, 0, BUFSIZE)) != -1) {
          pc_strm.write(spcbuf, 0, ii);
        }
        pc_strm.flush();
      } catch (Exception e) {
        err(3);  // err(4) should be called, nakov rlz
        pssck.close();
        return;
      }

      cpsck.close();
      System.out.println(cstr + " : closed (0)\n");

      pssck.close();
    } catch (IOException e) {
      System.err.println("IOerr: " + e.getMessage());
    }
  }
}

public class SPS {
  public static void main(String[] argv) {
    ServerSocket ss;
    Socket sock;
    ClntThrd cl;

    try {
      ss = new ServerSocket(50505);

      System.out.println("listening on " + ss.getInetAddress().getHostAddress() + ":" +
                         ss.getLocalPort());

      while (true) {
        sock = ss.accept();
        cl = new ClntThrd(sock);
        cl.start();
      }

    } catch (Exception e) {
      System.err.println("err: " + e.getMessage());
    }
  }
}

/* EOF */
