
//Title:        Simple Mail  Server, 2002, Inet-Java
//Version:      
//Copyright:    Copyright (c) 1999
//Author:       Vasil Karpachev
//Company:      fmi
//Description:  This project is for the Internet Programming in Java, 
//for the summer semester of the 2002 year.
//It should implement both POP3 on 110 port
//and SMTP on 25 port with the subset of 
//the commands in the above mentioned protocols.
//Its aim is to exercise java.net.* and java.lang.Thread
package SimpleMailServer;
import java.net.*;
import java.io.*;


/**
  *  Contains data and methods common to  SMSpop3ServerThread
  *  and  SMSsmtpServerThread, which are responsible
  *  for "managing the conversation" to POP3 and SMTP
  *  clients.
  *
  *  @see  SimpleMailServer.SMSpop3ServerThread
  *  @see  SimpleMailServer.SMSsmtpServerThread
  */
abstract class SMSAbstractServerThread extends Thread
{
	/** the server socket to the client. The client socket
        * and this socket talk by the rules of the POP3
        * protocol. See RFC 1939 for info about POP3
        */
	protected Socket m_socket;

      /** for input && ouput with a client */
      protected BufferedReader in;
      /** for input && ouput with a client */
	protected PrintWriter out;



      /**
        * Flag indicating if this thread should kill itself.
        * I.e. to commit suicide, or harakiri, or maybe
        * some sort of kamikadze.. ;))
        * <BR><BR>
        * -) if is_still_living==true then this thread should
        * continue to live and work <BR>
        * -) if is_still_living==false it should end his
        * hard and wong endured live.
        */
      protected boolean m_still_living= true;
      /**
        * Flag indicating if this thread is currently
        * carying conversation. If it is NOT, then
        * this thread sleeps, while waiting another
        * request for POP3 connection to become available.
        *
        * This may happen when SMSServerThreadManager decides
        * that this thread should not be killed(beacause
        * creating and killing threads is time and CPU
        * consuming process).
        */
      protected boolean m_active= false;

      /** for debug information messages */
      protected SMSMessageTracer m_logger= new SMSMessageTracer(this.toString());


	/**
        *  Create alive && not working thread
        */
      abstract SMSAbstractServerThread createThread();

	/**
        *  Create alive && working thread
        */
      abstract SMSAbstractServerThread createThread(Socket s);

      /**
        *   Change connection so that this thread can
        *   work with onother client. Set working flag.
        */
      abstract void changeConnection(Socket s) throws  SMSThreadBusyException;

      /** determines if this thread is now carrying
        * conversation with a client
        */
      abstract boolean isActive();

      /** change the active(working) state of this thread */
      abstract void setActiveState(boolean flag);

      /** determines if this thread should stop his existance */
	abstract boolean isDead();

      /** sets if this thread should be killed in the
        * first possible momment.
        */
      abstract void setAliveState(boolean flag);



      /**
        *  set up input and output streams
        */
      protected boolean initConnection()
      {
      	m_logger.print("initiating conversation");
	      try {
       	  in =
            	new BufferedReader(
                	  new InputStreamReader(
                    	m_socket.getInputStream()));
            } catch (IOException e)
            { m_logger.error("unable to open output stream to client");
              return false;            }
     	      try {
               // Enable auto-flush:
               out =
               	new PrintWriter(
                	  new BufferedWriter(
                      new OutputStreamWriter(
                        m_socket.getOutputStream())), true);
            } catch (IOException e)
            { m_logger.error("unable to open input stream to client");
              return false; }

            return true;
      }

      /**
        *  close input and output streams
        */
      protected void closeConnection()
      {    //close connection
            try {
            	m_socket.close();
            } catch(IOException e)
      	{ 	m_logger.error("cannont close connection "+e); 	}
            m_logger.print("connection closed succesfuly");
            m_socket= null;

      	m_active= false; //not active- need rest ;)
      }
}
