
//Title:        Simple Mail  Server, 2002, Inet-Java
//Version:      
//Copyright:    Copyright (c) 2002
//Author:       Vasil Karpachev
//Company:      fmi
//Description:  This project is for the Internet Programming in Java, 
//for the summer semester of the 2002 year.
//It should implement both POP3 on 110 port
//and SMTP on 25 port with the subset of 
//the commands in the above mentioned protocols.
//Its aim is to exercise java.net.* and java.lang.Thread
package SimpleMailServer;
import java.lang.Thread;
import java.net.Socket;
import java.net.ServerSocket;
import java.io.*;
import java.util.Vector;
import SimpleMailServer.*;

/**
  * This class will be responsible for managing
  * SMSpop3ServerThread objects. <BR>
  * <BR>
  * SMSServerThreadMenager keeps a number of SMSpop3ServerThread
  * threads alive. When request for connection arrives this
  * class check if there is "free" SMSpop3ServerThread thread and
  * gives it the socket to carry conversation. <BR>
  * <BR>
  * This feature (izgazica) was implemented to make the
  * project interesting and because PEOPLE SAY that in
  * this way it is faster. I.e. creating new threads and
  * killing them is time and CPU consuming.. just imagine
  * 1000 request for connection(like mail.yahoo.com),
  * then 1000 threads should be created..
  * 
  */
class SMSServerThreadMenager extends Thread
{
 	/**
        * Positive integer indicating how many
        * threads should be kept alive, even when
        * there are no so many pop3 connections.
        *
        * This feature is feasible, because thread
        * createion and destruction is time and
        * CPU consuming process. In this way threads
        * could be reused.
        */
      protected int m_MIN_NUMBER_ALIVE_THREADS= 5;

      /**
        * maximum allowed number of threads.
        */
      protected int m_MAX_NUMBER_ALIVE_THREADS= 30;

      /**
        * keeping track of all SMSAbstractServerThread threads
        */
      protected Vector m_thread_vector;

      /** number of active threads */
      protected int m_num_active_threads= 0;


      /**
        * Collection of the java.net.Sockets that wait to cary
        * conversation with different clients.
        */
      protected Vector m_socket_vector= new Vector();

      void addSocket(Socket s) {
      	m_socket_vector.add(s);   //synchronized by Vector
      }

      /**
        * used to create SMSpop3ServerThread or
        * SMSsmptServerThread.
        * See SMSAbstractServerThread.createThread
        */
      private SMSAbstractServerThread m_thread_factory= null;

      /**
        * parameter indicating how many threads should
        * be kept alive at minimum.
        */
	SMSServerThreadMenager(int num_alive,
      					SMSAbstractServerThread thrd)   {
		m_MIN_NUMBER_ALIVE_THREADS= num_alive;
            init(thrd);
	}

      /** specify number of *live threads* and number of
        * maximum threads.
        */
	SMSServerThreadMenager(int num_alive,
      					int max_alive,
                                    SMSAbstractServerThread thrd)   {
		m_MIN_NUMBER_ALIVE_THREADS= num_alive;
            m_MAX_NUMBER_ALIVE_THREADS= max_alive;
            init(thrd);
	}

      /**
        * use the deafult values.
        */
	SMSServerThreadMenager(SMSAbstractServerThread thrd)
      {  init(thrd);	}

      private void init(SMSAbstractServerThread thrd) {
      	m_thread_factory= thrd; //set factory
            m_thread_vector= new Vector();

      	if (m_MAX_NUMBER_ALIVE_THREADS<m_MIN_NUMBER_ALIVE_THREADS)
            	m_MAX_NUMBER_ALIVE_THREADS= m_MIN_NUMBER_ALIVE_THREADS+10;

            // start m_number_alive_threads threads to be ready to serv
      	for (int i=0;i<m_MIN_NUMBER_ALIVE_THREADS;i++)  {
            	SMSAbstractServerThread thread= m_thread_factory.createThread();
                  thread.start(); //live && !working
            	m_thread_vector.add(thread);
            }

	}

      /** if this manager is working */
      protected boolean m_active= false;

	/** determines if this thread is now talking */
	public boolean isActive()
      { return m_active; }
      public void setActiveState(boolean flag)
      { m_active= flag; }


      protected boolean m_still_living= true;

      /** determines if this thread should stop his existance */
      boolean isDead()
      { return !m_still_living; }

      /** sets if this thread should stop breathing in the
        * first possible momment
        */
      void setAliveState(boolean flag)
      { m_still_living= flag; }



      /** SMTP or POP3 thread manager */
      private String m_manages_who;
      void setMenagesString(String w)
      { w= "("+w+") "; }
      /** for debug information messages */
      private SMSMessageTracer m_logger= new SMSMessageTracer("THREAD MANAGER");

      /**
        * manages it threads
        */
      public void run()
      {
      	if (m_manages_who!=null)
            	m_logger.addToPrefix(m_manages_who);
            m_logger.print("successfuly started");

            // while stil alive
      	while (m_still_living)
            {
                  while (!m_active)  {
                  	if (m_socket_vector.size()>0)
                        	break; //work needs to be done
                  	try {
  					sleep(500);
                        } catch (InterruptedException e)
                        {m_active=true;}
                  }

                  if (isDead()) return; //THE END of this manager

                  handleNewSocketsAndThreads();
            }
      }

      /**
        * -) check if there are free threads <BR>
        * -) create a new thread if needed <BR>
        * -) kill some threads if too many
        */
      void handleNewSocketsAndThreads()
      {
      	int num_new_sockets= m_socket_vector.size();
            if (num_new_sockets==0) return; //do not have work here

            int num_threads_not_active= 0;
            int num_all_threads=  m_thread_vector.size();
            // find  the number of threads that are *not active*
            for (int i=0; i<num_all_threads; i++) {
            	SMSAbstractServerThread thread=
                  	(SMSAbstractServerThread )m_thread_vector.elementAt(i);
            	if (!thread.isActive())
                         num_threads_not_active++;
            }

            // how many threads to delete?
            int num_threads_to_del= num_threads_not_active - num_new_sockets;

            int delta= m_num_active_threads - m_MIN_NUMBER_ALIVE_THREADS;
            if (num_threads_to_del > delta)
                 	{ num_threads_to_del= (delta>0)?delta:0; }
            if (num_threads_to_del < 0) num_threads_to_del= 0;

            // REMOVE unnecessary threads
            for (int i=0; i<num_all_threads; i++)
            {
            	SMSAbstractServerThread thread=
                  	(SMSAbstractServerThread )m_thread_vector.elementAt(i);
            	if (!thread.isActive()) { //not active
                  	if (num_threads_to_del>0) { //needs to be deleted
                        	num_threads_to_del--;
                              thread.setAliveState(false);//kill it
                              m_thread_vector.remove(i);
                        } else break; //stop the iteration

                  }

            }

            // distribute sockets among   not active  threads
            for (int i=0; i<num_all_threads; i++)
            {
            	SMSAbstractServerThread thread=
                  	(SMSAbstractServerThread )m_thread_vector.elementAt(i);
            	if (!thread.isActive()) { //not active
                        if (num_new_sockets>0) {
                        	num_new_sockets--;
                              Socket s=
                              (Socket)m_socket_vector.elementAt(num_new_sockets);
                              try {
                              	thread.changeConnection( s );
                              } catch(SMSThreadBusyException e)
                              {   m_logger.print("trying to disturb busy thread..");
                                  num_new_sockets++; //try again with the next thread
                                  continue;
                              }
                              thread.setActiveState(true);
                              thread.interrupt();
                        } else break; //stop the iteration
                  }
            }

            // create new trreads if necessary
            for (num_new_sockets--; num_new_sockets>=0; num_new_sockets--)
            {
               if (m_thread_vector.size()<m_MAX_NUMBER_ALIVE_THREADS) {
            	Socket s= (Socket)m_socket_vector.elementAt(num_new_sockets);
                  SMSAbstractServerThread thread=
                  	m_thread_factory.createThread(s); //assing new thread to carry conversation
            	m_thread_vector.add(thread);
                  thread.start(); //let it go
               } else {
               	m_logger.error("maximum thread limit reached!");
                  m_logger.error("rejecting the request(s) for connection");
                    for (int i=0; i<m_socket_vector.size(); i++)
                    {	try {
                          //close connection
                  	  ((Socket)m_socket_vector.elementAt(i)).close();
                        } catch(IOException e)
                        { m_logger.error("cannont close connection "+e); }
                    }
               }
            }

            m_socket_vector.clear();
            m_active= false; //has done his work - let him rest
      }
}