
//Title:        Simple Mail  Server, 2002, Inet-Java
//Version:
//Copyright:    Copyright (c) 2002
//Author:       Vasil Karpachev
//Company:      fmi
//Description:  This project is for the Internet Programming in Java,
//for the summer semester of the 2002 year.
//It should implement both POP3 on 110 port
//and SMTP on 25 port with the subset of
//the commands in the above mentioned protocols.
//Its aim is to exercise java.net.* and java.lang.Thread
package SimpleMailServer;
import java.lang.Thread;
import java.net.Socket;
import java.net.ServerSocket;
import java.io.*;

/**
  * Represents the thread that implements POP3 server.
  *
  * This class sits around and listens for new requests
  * for pop3 connections. When connection arrives it gives
  * the socket to the SMSServerThreadMenager.
  */
class SMSpop3Server extends Thread {

	/** indicates if the run method should be exited,
        * i.e. kill this thread
        */
	protected boolean m_still_alive= true;

      /** the manager for the threads */
      protected SMSServerThreadMenager m_manager =
      		new SMSServerThreadMenager(new SMSpop3ServerThread());

	SMSpop3Server() {
	}

      /** indicated that POP3 server should exit */
      void shutDownServer() {
      	m_still_alive= false;
      }


      /** for debug information messages */
      private SMSMessageTracer m_logger= new SMSMessageTracer("POP3 SERVER");


      /**
        * listen for new connections. When connection arrives it gives
        * the socket to the SMSServerThreadMenager.
        */
      public void run()
      {
      	m_logger.print("successfuly started");
            m_manager.setMenagesString("POP3");
            m_manager.start(); //let it go

           	ServerSocket server;
           	try {
           		server= new ServerSocket(SMSMain.POP3_PORT);
            } catch (IOException e)
            { 	m_logger.error("failed to create server socket!"+e);
                 	return;	}

		m_logger.print("server socket created");

            // listen for connection till alive
            while (m_still_alive)
            {
                  Socket socket;
                  try  {
                  	socket= server.accept();
                  } catch (IOException e)
                  {  m_logger.error("failed to accept socket!"+e);
                     return;                 	}

			m_logger.print("connect request");

                  //add socket to the menager
			m_manager.addSocket(socket);
                  //wake him up ;), if asleep
                  m_manager.interrupt();  // process it
            }

            m_logger.print("bye");
            m_manager.setAliveState(false); //kill manager
      }
}