//Title:        Simple Mail  Server, 2002, Inet-Java
//Version:
//Copyright:    Copyright (c) 2002
//Author:       Vasil Karpachev
//Company:      fmi
//Description:  This project is for the Internet Programming in Java,
//for the summer semester of the 2002 year.
//It should implement both POP3 on 110 port
//and SMTP on 25 port with the subset of
//the commands in the above mentioned protocols.
//Its aim is to exercise java.net.* and java.lang.Thread
package SimpleMailServer;
import java.net.*;
import java.io.*;
import java.util.Vector;

/**
  *  This class represent the part of the Simple Mail Server
  *  that is responsible for SMTP protocol server side support.
  */
class SMSsmtpServerThread extends SMSAbstractServerThread
{
	/** determines if this thread is now talking */
	public boolean isActive()
      { return m_active; }
      public void setActiveState(boolean flag)
      { m_active= flag; }

      /**
        * Flag indicating if this thread should kill itself.
        * I.e. to commit suicide, or harakiri, or maybe
        * some sort of kamikadze.. ;))
        * <BR><BR>
        * -) if is_still_living==true then this thread should
        * continue to live and work <BR>
        * -) if is_still_living==false it should end his
        * hard and wong endured live.
        */
      protected boolean m_still_living= true;

      /** determines if this thread should stop his existance */
      boolean isDead()
      { return !m_still_living; }

      /** sets if this thread should stop breathing in the
        * first possible momment
        */
      void setAliveState(boolean flag)
      { m_still_living= flag; }

     SMSAbstractServerThread createThread()
     { return new SMSsmtpServerThread(); }

     SMSAbstractServerThread createThread(Socket s)
     { return new SMSsmtpServerThread(s);   }

      /**
        * Consturcts a thread with *in* and *out* sockets
        * for the pop3 conversation. The initial sate
        * of the thread is *living* and *active*.
        */
      SMSsmtpServerThread(Socket s)
      {
      	m_socket= s;
            setAliveState(true); //living
            setActiveState(true); //should begin work
      }

      /**
        * Consturcts a thread no sockets.
        * The initial sateof the thread is *living*
        * and *not active*.
        */
      SMSsmtpServerThread()
      {
      	m_socket= null;
            setAliveState(true); //living
            setActiveState(false); //still not working
      }

      /**
        * Changes the *in* and *out* connections.
        * This will be usefull if SMSpop3Server decides
        * that this thread should not die, but instead
        * sleep and when new request for connection arrives
        * SMSpop3Server sets the new connection  and
        * interrupts this thread so it can continue to
        * process it immediately.
        */
      void changeConnection(Socket s)  throws  SMSThreadBusyException
      {
      	if (!isActive())  { //if not caring converstion
            	m_socket= s;
            } else {
            	throw new SMSThreadBusyException(); /*error*/
            }

      }

      /** for debug information messages */
      private SMSMessageTracer m_logger= new SMSMessageTracer(this.toString());


      public void run()
      {
		while (m_still_living)
            {
                  while (!m_active)  {
                  	m_logger.printRandomChar(hashCode());
                  	try {
  					sleep(500);
                        } catch (InterruptedException e)
                        {m_active=true;}
                  }
                  if( isDead() ) return; //THE END

                  carySMTPConversation();
            }
      }




      private void carySMTPConversation()
      {
      	boolean ok;
            ok= initConnection();
            if (!ok)
            {	closeConnection(); return;     }

            out.println("220 Simple mail server");
            out.flush();

            String helo= 		"HELO ";
            String mail_from= 	"MAIL FROM:";
            String rcpt_to= 		"RCPT TO:";
            String data= 		"DATA";
            String quit= 		"QUIT";

            Vector rcptVec= new Vector(); //recepients vector

            while (true)
            {
	            String command;
            	try {
            		command= in.readLine();
                  } catch (IOException e) {
                  	m_logger.print("read error");
                        closeConnection(); return;
                  }

/*
   3.5.  OPENING AND CLOSING

      At the time the transmission channel is opened there is an
      exchange to ensure that the hosts are communicating with the hosts
      they think they are.

      The following two commands are used in transmission channel
      opening and closing:

         HELO <SP> <domain> <CRLF>

         QUIT <CRLF>

      In the HELO command the host sending the command identifies
      itself; the command may be interpreted as saying "Hello, I am
      <domain>".

      -------------------------------------------------------------

                     Example of Connection Opening

         R: 220 BBN-UNIX.ARPA Simple Mail Transfer Service Ready
         S: HELO USC-ISIF.ARPA
         R: 250 BBN-UNIX.ARPA

                               Example 5

      -------------------------------------------------------------

      -------------------------------------------------------------

                     Example of Connection Closing

         S: QUIT
         R: 221 BBN-UNIX.ARPA Service closing transmission channel

                               Example 6

      -------------------------------------------------------------
*/
                  if (command.equals(quit)) {
            		out.print("221 Bye");
            		out.flush();
                  	break; //end of conversation
                  }

			if (command.length() > helo.length() &&
                  		command.substring(0, helo.length())
                              	.equals(helo)  &&
                                    command.charAt(helo.length()-1)== ' '
                                    			)
                  {
                       String reply=  "250 Welcome ";
                       String hostname= command.substring(helo.length());
                       reply+= hostname;

                       out.println(reply);

                       out.flush();
                       continue;
                  }
/*
      There are three steps to SMTP mail transactions.  The transaction
      is started with a MAIL command which gives the sender
      identification.  A series of one or more RCPT commands follows
      giving the receiver information.  Then a DATA command gives the
      mail data.  And finally, the end of mail data indicator confirms
      the transaction.
      -------------------------------------------------------------

                     Example of the SMTP Procedure

         This SMTP example shows mail sent by Smith at host Alpha.ARPA,
         to Jones, Green, and Brown at host Beta.ARPA.  Here we assume
         that host Alpha contacts host Beta directly.

            S: MAIL FROM:<Smith@Alpha.ARPA>
            R: 250 OK

            S: RCPT TO:<Jones@Beta.ARPA>
            R: 250 OK

            S: RCPT TO:<Green@Beta.ARPA>
            R: 550 No such user here

            S: RCPT TO:<Brown@Beta.ARPA>
            R: 250 OK

            S: DATA
            R: 354 Start mail input; end with <CRLF>.<CRLF>
            S: Blah blah blah...
            S: ...etc. etc. etc.
            S: <CRLF>.<CRLF>
            R: 250 OK

         The mail has now been accepted for Jones and Brown.  Green did
         not have a mailbox at host Beta.

                               Example 1

      -------------------------------------------------------------
*/

			if (command.length() > mail_from.length() &&
                  		command.substring(0, mail_from.length())
                              	.equals(mail_from)			)
                  {
/*
         The first step in the procedure is the MAIL command.  The
         <reverse-path> contains the source mailbox.

            MAIL <SP> FROM:<reverse-path> <CRLF>

         This command tells the SMTP-receiver that a new mail
         transaction is starting and to reset all its state tables and
         buffers, including any recipients or mail data.  It gives the
         reverse-path which can be used to report errors.  If accepted,
         the receiver-SMTP returns a 250 OK reply.

         The <reverse-path> can contain more than just a mailbox.  The
         <reverse-path> is a reverse source routing list of hosts and
         source mailbox.  The first host in the <reverse-path> should be
         the host sending this command.


*/
                       String reply=  "250 ";
                       String from= command.substring(mail_from.length());
                       int beg= from.indexOf('<');
                       int end= from.indexOf('>');
                       if (end>0 &&
                       		(beg==0 || beg==1) && beg<end-1) {
	                       from= from.substring(beg+1,end);
                       } else {
                       		from= from.trim();//"unknown";
                       }


                       reply+= from + " Sender OK";

                       rcptVec.clear();
                       out.println(reply);

                       out.flush();
                       continue;
                  }


			if (command.length() > rcpt_to.length() &&
                  		command.substring(0, rcpt_to.length())
                              	.equals(rcpt_to)			)
                  {
/*
         The second step in the procedure is the RCPT command.

            RCPT <SP> TO:<forward-path> <CRLF>

         This command gives a forward-path identifying one recipient.
         If accepted, the receiver-SMTP returns a 250 OK reply, and
         stores the forward-path.  If the recipient is unknown the
         receiver-SMTP returns a 550 Failure reply.  This second step of
         the procedure can be repeated any number of times.

         The <forward-path> can contain more than just a mailbox.  The
         <forward-path> is a source routing list of hosts and the
         destination mailbox.  The first host in the <forward-path>
         should be the host receiving this command.

*/
                       String reply=  "250 ";
                       String from= command.substring(rcpt_to.length());
                       String to= "";

                       int end= from.indexOf('>');
                       int beg= from.indexOf('<');

                       if (end>0 && (beg==0 || beg==1) && beg<end-1)
                       {
	                       to= from.substring(beg+1,end);
                       } else {
                       		/*String err_reply= "550 ";
                              err_reply+= from;
                              err_reply+= " Recipient not found on this server";
                              out.println(err_reply);

                              out.flush();
                              continue;*/
                              to= from;
                              to= to.trim();
                       }

                       if (!to.toUpperCase().equals(SMSMain.Users[0].toUpperCase()) &&
                       		!to.toUpperCase().equals(SMSMain.Users[1].toUpperCase()) &&
                              !to.toUpperCase().equals(SMSMain.Users[2].toUpperCase())     )
                       {
                                String err_reply="550 "
                                		+ to + " not found on this server";
                             	  out.println(err_reply);

                                out.flush();
                                continue;
                       }

                       //to= to.toLowerCase();
                       reply+= to + " Recipient OK";

                       if ( !rcptVec.contains(to) )
                       		rcptVec.addElement(to.toLowerCase());
                       out.println(reply);

                       out.flush();
                       continue;
                  }


                   if (command.length() == data.length() &&
                  		command.equals(data)			)
/*
         The third step in the procedure is the DATA command.

            DATA <CRLF>

         If accepted, the receiver-SMTP returns a 354 Intermediate reply
         and considers all succeeding lines to be the message text.
         When the end of text is received and stored the SMTP-receiver
         sends a 250 OK reply.

         Since the mail data is sent on the transmission channel the end
         of the mail data must be indicated so that the command and
         reply dialog can be resumed.  SMTP indicates the end of the
         mail data by sending a line containing only a period.  A
         transparency procedure is used to prevent this from interfering
         with the user's text (see Section 4.5.2).

            Please note that the mail data includes the memo header
            items such as Date, Subject, To, Cc, From [2].

         The end of mail data indicator also confirms the mail
         transaction and tells the receiver-SMTP to now process the
         stored recipients and mail data.  If accepted, the
         receiver-SMTP returns a 250 OK reply.  The DATA command should
         fail only if the mail transaction was incomplete (for example,
         no recipients), or if resources are not available.

*/
                  {
                        if (rcptVec.size()==0) // RCPT TO: not finished succssfuly
                        {
                        	out.println("503 Invalid state");
                              out.flush();
                              continue;
                        }

                        // RCPT TO: finished succssfuly
                        out.println("354 Enter mail, end with \".\" on a line by itself");
                        out.flush();

                        Vector mail= new Vector();
                        String line;
                        SMSMessage message= new SMSMessage();


                        do {
            			try {
            				line= in.readLine();
                  		} catch (IOException e) {
                  			m_logger.print("read error");
                        		closeConnection(); return;
                  		}

                              message.appendLine(line);

                        } while (!line.equals("."));


                        for (int i=0; i<rcptVec.size(); i++)
                        {
                        	if(  ((String)rcptVec.elementAt(i))
                                    	.equals("user1")		    )
                              	SMSMain.user1_BOX.addMessage(message);
                        	if(  ((String)rcptVec.elementAt(i))
                                    	.equals("user2")		    )
                              	SMSMain.user2_BOX.addMessage(message);
                        	if(  ((String)rcptVec.elementAt(i))
                                    	.equals("user3")		    )
                              	SMSMain.user3_BOX.addMessage(message);
                        }

                       out.println("250 Message delivered");

                       rcptVec.clear();
                       out.flush();
                       continue;
                  }

		out.println("500 Invalid command");

            }

            closeConnection();
      }

}