package client;

/**
 * FTP Client
 * Project 1.5, 'Internet Programming With Java' Cource
 * 
 * @author Vesselin Mitrov, vmitrov@bulgaria.com
 */
 
import java.util.*;
import java.io.*;
import java.net.*;
 
public class Client {
  
  private Socket socket = null;
  private InputStream in = null;
  private OutputStream out = null;
  private BufferedReader consoleIn = null;
  
  public Client() {
    consoleIn = new BufferedReader(new InputStreamReader(System.in));
    start();
  }
  
  public static void main(String[] args) {
    new Client();
  }
  
  private void start() {
    try {
      Properties props = new Properties();
      props.load(new FileInputStream("client.properties"));
      String addr = props.getProperty("server");
      int portNumber = (new Integer(props.getProperty("port"))).intValue();
      socket = new Socket(addr, portNumber);
      in = socket.getInputStream();
      out = socket.getOutputStream();
      while (true) {
        System.out.print("> ");
        String str = consoleIn.readLine();
        if (str.equals("")) {
          // nothing to do
        } else if (str.equals("DIR") || str.equals("LIST") || str.equals("LS") || str.equals("HELP")) {
          sendString(str);
          receiveMultiple();
        } else if (str.startsWith("CD ")) {
          sendString(str);
          receiveSingle();
        } else if (str.startsWith("GET ")) {
          sendString(str);
          receiveFile();
        } else if (str.startsWith("UPLOAD ")) {
          uploadFile(str);
        } else if (str.equals("QUIT")) {
          sendString(str);
          receiveSingle();
          break;
        } else {
          wrongCommand(str);
        }
      }
    } catch (Exception exc) {
      throw new RuntimeException(exc.toString());
    } finally {
      try {
        consoleIn.close();
        socket.close();
      } catch (Exception e) {
      }
    }
  }
  
  private void receiveMultiple() throws Exception {
    String str = readString();
    while (!str.equals(".")) {
      System.out.println(str);
      str = readString();
    }
    System.out.println(".");
  }
  
  private void receiveSingle() throws Exception {
    System.out.println(readString());
  }
  
  private void receiveFile() throws Exception {
    String result = readString();
    System.out.println(result);
    if (result.startsWith("-")) {
      return;
    }
    int len = Integer.parseInt(readString());
    System.out.println(len);
    
    for (int i = 0; i < len; i++) {
      System.out.print((char)in.read());
    }
    System.out.println();
  }
  
  private void uploadFile(String str) throws Exception {
    FileInputStream fileIn = null;
    try {
      File file = new File(str.substring(7).trim());
      String fileName = file.getName();
      long bytes = file.length();
      fileIn = new FileInputStream(file);
      sendString("SEND " + fileName);
      sendString(String.valueOf(bytes));
      
      for (long i = 0; i < bytes; i++) {
        out.write(fileIn.read());
      }
    } finally {
      if (fileIn != null) {
        fileIn.close();
      }
    }
    
    receiveSingle();
  }
  
  private void sendString(String str) throws IOException {
    str += "\r\n";
    byte[] bytes = str.getBytes();
    out.write(bytes);
  }
  
  private String readString() throws IOException {
    StringBuffer str = new StringBuffer();
    while (true) {
      int b = in.read();
      if (b == 13) {
        int b2 = in.read();
        if (b2 == 10) {
          break;
        } else {
          str.append((char)b);
          str.append((char)b2);
        }
      } else {
        str.append((char)b);
      }
    }
    return str.toString();
  }
  
  private void wrongCommand(String str) {
    System.out.println();
    System.out.println("No such command: " + str + ". Type HELP to see available commands.");
    System.out.println();
  }
}