package ftpserver;

/**
 * ServerThread
 * Project 1.5, 'Internet Programming With Java' Cource
 * 
 * @author Vesselin Mitrov, vmitrov@bulgaria.com
 */
 
import java.io.*;
import java.net.*;
 
public class ServerThread extends Thread {
  
  private String root = null;
  private String currentDir = null;
  private Socket socket = null;
  
  private InputStream in = null;
  private OutputStream out = null;
  
  private char sepChar = '\\';
  
  public ServerThread(String _root, Socket _socket) {
    root = _root;
    currentDir = "" + sepChar;
    socket = _socket;
    try {
      in = socket.getInputStream();
      out = socket.getOutputStream();
    } catch (IOException ioexc) {
      throw new RuntimeException(ioexc.toString());
    }
    
    sepChar = File.separatorChar;
  }
  
  public void run() {
    try {
      while (true) {
        String command = readString();
        if (command.equals("DIR") || command.equals("LIST") || command.equals("LS")) {
          commandDir();
        } else if (command.startsWith("CD ")) {
          commandCd(command.substring(2).trim());
        } else if (command.equals("HELP")) {
          commandHelp();
        } else if (command.startsWith("GET")) {
          commandGet(command.substring(3).trim());
        } else if (command.startsWith("SEND")) {
          commandSend(command.substring(5));
        } else if (command.equals("QUIT")) {
          sendString("+Bye");
          break;
        }
      }
    } catch (IOException ioexc) {
      ioexc.printStackTrace();
      // nothing to do
    } finally {
      try {
        socket.close();
      } catch (IOException exc) {
      }
    }
  }
  
  private void commandDir() throws IOException {
    File current = new File(root + currentDir);
    File[] allFiles = current.listFiles();
    int all = 0;
    int allDirs = 0;
    long allBytes = 0;
    sendString("+Listing directory " + currentDir);
    sendString("");
    if (allFiles.length == 0) {
      sendString("Total 0 files (0 bytes), 0 subdirectories.");
      sendString(".");
      return;
    }
    for (int i = 0; i < allFiles.length; i++) {
      String entry = allFiles[i].getName() + "\t";
      if (allFiles[i].isFile()) {
        entry += allFiles[i].length();
        all++;
        allBytes += allFiles[i].length();
      } else {
        entry += "<DIR>";
        allDirs++;
      }
      sendString(entry);
    }
    sendString("");
    sendString("Total " + all + " files (" + allBytes + " bytes), " + allDirs + " subdirectories.");
    sendString(".");
  }
  
  private void commandCd(String destination) throws IOException {
    if (destination.equals("..")) {
      if (currentDir.equals(String.valueOf(sepChar))) {
        sendString("-Invalid directory.");
      } else {
        int strLength = currentDir.length();
        if ((strLength > 1) && (currentDir.charAt(strLength - 1) == sepChar)) {
          currentDir = currentDir.substring(0, strLength - 1);
        }
        int index = currentDir.lastIndexOf(sepChar);
        if (index == 0) {
          currentDir = "" + sepChar;
        } else {
          currentDir = currentDir.substring(0, index);
        }
        sendString("+Directory changed to " + currentDir + ".");
      }
    } else {
      if (!currentDir.endsWith("" + sepChar)) {
        currentDir += sepChar;
      }
      File newDir = new File(root + currentDir + destination);
      if (newDir.exists() && !newDir.isFile()) {
        currentDir += destination;
        sendString("+Directory changed to " + currentDir + ".");
      } else {
        sendString("-Invalid directory.");
      }
    }
  }
  
  private void commandGet(String fileName) throws IOException {
    if (!currentDir.endsWith("" + sepChar)) {
      currentDir += sepChar;
    }
    File file = new File(root + currentDir + fileName);
    if (!file.exists()) {
      sendString("-No such file: " + fileName);
    } else {
      sendString("+File size and content follow:");
      sendString(String.valueOf(file.length()));
      FileInputStream fileStream = null;
      try {
        fileStream = new FileInputStream(file);
      } catch (FileNotFoundException exc) {
        // not possible
      }
      
      int i;
      while ((i = fileStream.read()) != -1) {
        out.write(i);
      }
      fileStream.close();
    }
  }
  
  private void commandSend(String fileName) throws IOException {
    long bytes = Long.parseLong(readString());
    File file = null;
    FileOutputStream fileOut = null;
    boolean exists = false;
    try {
      if (!currentDir.endsWith("" + sepChar)) {
        currentDir += sepChar;
      }
      file = new File(root + currentDir + fileName);
      exists = file.exists();
      fileOut = new FileOutputStream(file);
    } catch (IOException ioexc) {
      ioexc.printStackTrace();
      sendString("-Can not store file in " + currentDir + ".");
      return;
    }
    try {
      if (exists) {
        sendString("-Can not store file in " + currentDir + ".");
        return;
      }
      
      for (long i = 0; i < bytes; i++) {
        fileOut.write(in.read());
      }
      
      sendString("+File " + fileName + " stored.");
    } finally {
      fileOut.close();
    }
  }
  
  private void commandHelp() throws IOException {
    sendString("");
    sendString("Valid commands:");
    sendString("  DIR - lists the current directory");
    sendString("  CD directory - changes current directory");
    sendString("  GET filename - retrieves given file from current directory");
    sendString("  SEND filename");
    sendString("       filesize");
    sendString("       <file contents> - sends given file to the server");
    sendString("  HELP - displays this message");
    sendString("  QUIT - closes the session");
    sendString(".");
  }
  
  private void sendString(String str) throws IOException {
    str += "\r\n";
    byte[] bytes = str.getBytes();
    out.write(bytes);
  }
  
  private String readString() throws IOException {
    StringBuffer str = new StringBuffer();
    while (true) {
      int b = in.read();
      if (b == 13) {
        int b2 = in.read();
        if (b2 == 10) {
          break;
        } else {
          str.append((char)b);
          str.append((char)b2);
        }
      } else {
        str.append((char)b);
      }
    }
    return str.toString();
  }
}