
import java.io.*;
import java.net.*;
import java.util.*;
/*
Configuration class
*/
public class Config
{
	// WebServer class
	public static final int WEBSERVER_DEFAULT_PORT = 80;
	public static final int WEBSERVER_MAXCONNECTIONS = 256;
	public static final String WEBSERVER_DEFAULT_ROOT = "/";

	// Connection class
	public static final String
		CONNECTION_RESPONSE_NOT_IMPLEMENTED =
			"HTTP/1.0 501 Not Implemented\n" +
			"MIME-Version: 1.0\n" +
			"Content-type: text/html\n" +
			"\n" +
			"<html> The requested command is not implemented ! </html>",
		CONNECTION_RESPONSE_NOT_FOUND =
			"HTTP/1.0 404 Not Found\n" +
			"MIME-Version: 1.0\n" +
			"Content-type: text/html\n" +
			"\n" +
			"<html> The requested document not found ! </html>",
		CONNECTION_RESPONSE_HEADER =
			"HTTP/1.0 200 OK\n" +
			"MIME-Version: 1.0\n" +
			"Content-type: ",
		CONNECTION_RESPONSE_FOOTER = "\n\n";
	public static final String[] CONNECTION_CONTENT_TYPE =
		{ "text/html", "image/gif", "image/jpeg", "application/zip", "text/plain" };
	public static final String[] CONNECTION_FILE_TYPE =
		{ "html htm ", "gif ", "jpg jpe ", "zip rar arj " };
	public static final int CONNECTION_FILEBUFFER_SIZE = 16384;

	public static final void CONNECTION_RESPONSE_DIRECTORY( /*String host, */String dirname, PrintStream out, File[] file ) throws IOException
	{
		if( !dirname.endsWith( "/" ) ) dirname += '/';
		out.print(
			"HTTP/1.0 200 OK\n" +
			"MIME-Version: 1.0\n" +
			"Content-type: text/html\n" +
			"\n" +
			"" );
		out.print(
			"<html><title>Listing of " + dirname + " directory" /*+ " on " + host */+ "</title>" +
			"<head><style type='text/css'><!--\n" +
			"th { white-space:nowrap; font-family:'Courier new'; color:blue; font-weight:bold; text-align:left; }" +
			"td { white-space:nowrap; font-family:'Courier new'; color:black; text-align:left; }" +
			".a { white-space:pre; font-family:'Courier new'; color:blue; font-size:large; font-weight:bold; }" +
			"\n--></style></head>" +
			"<body>" +
			"<font class='a'>Listing of " + dirname + "</font><p>" +
			"<table><tr><th>Name</th><th>Size</th><th>Last modified<br></th></tr>" );
		String line =
			"<tr><td>" +
			dupChar( '-', 30 ) + "&nbsp;&nbsp;&nbsp;&nbsp;</td><td>" +
			dupChar( '-', 20 ) + "&nbsp;&nbsp;&nbsp;&nbsp;</td><td>" +
			dupChar( '-', 15 ) + "</td></tr>";
		out.print( line );
		long size = 0;
		String s = (new File( dirname )).getParent();
		if( s == null ) s = "";
		out.print( "<tr><td><a href='" /*+ "http://" + host */+ s + "'>..</a>" );
		Arrays.sort( file, new FileSorter() );
		for( int i = 0; i < file.length; i++ )
		{
			size += file[ i ].length();
			Calendar c = Calendar.getInstance();
			c.setTime( new Date( file[ i ].lastModified() ) );
			s = file[ i ].getName();
			if( s.length() > 30 ) s = s.substring( 0, 27 ) + "..>";
			out.print(
				"<tr><td><a href='" /*+ "http://" + host */+ dirname + file[ i ].getName() + "'>" + s + "</a>" +
				"</td><td>" + ( ( file[ i ].isDirectory() )? "&lt;DIRECTORY&gt;" : "" + file[ i ].length() ) +
				"</td><td>" + c.get( Calendar.YEAR ) + '/' + c.get( Calendar.MONTH ) + '/' + c.get( Calendar.DATE ) + "</td></tr>" );
		}
		out.print( line );
		out.print( "<tr><td>" + file.length + " files</td><td>" + size + " bytes</td><td></td></tr>" );
		out.print( "</table></body></html>" );
	}
	private static final String dupChar( char ch, int count )
	{
		if( count <= 0 ) return "";
		char[] s = new char[ count ];
		Arrays.fill( s, ch );
		return new String( s );
	}
}

class FileSorter implements Comparator
{
	public int compare( Object o1, Object o2 )
	{
		try
		{
			if( ((File)o1).isDirectory() )
			{
				if( ((File)o2).isDirectory() ) return ((File)o1).getName().compareTo( ((File)o2).getName() );
				return -1;
			}
			if( ((File)o2).isDirectory() ) return 1;
			return ((File)o1).getName().compareTo( ((File)o2).getName() );
		}
		catch( ClassCastException e ) { }
		return 0;
	}
}