import java.io.*;
import java.net.*;
import java.util.*;

/*
Handles a HTTP request
*/
public class Connection extends Thread
{
	private String wwwroot;
	private Socket socket;
	private HTTPRequest request;

	// Constructor:
	//   parent		- the thread's group
	//   socket		- connection socket
	//	 wwwroot	- web service root
	public Connection( ThreadGroup tgroup, Socket socket, String wwwroot )
	{
		super( tgroup, socket.getInetAddress().getHostName() + ":" + socket.getPort() );
		this.socket = socket;
		this.wwwroot = wwwroot;
	}
	public void run()
	{
		InputStream binput;
		BufferedReader tinput;
		OutputStream boutput;
		PrintStream toutput;
		try
		{
			binput = socket.getInputStream();
			boutput = socket.getOutputStream();
			tinput = new BufferedReader( new InputStreamReader( binput ) );
			toutput = new PrintStream( boutput );

			request = getRequest( tinput );
			sendResponse( toutput, boutput );

			socket.close();
		}
		catch( Exception e )
		{
			try	{ socket.close(); } catch( Exception e1 ) { }
			return;
		}
	}
	public String toString()
	{
		return socket.getInetAddress().getHostAddress() + ':' + socket.getPort() + "\t\t" + request.resource;
	}
	public void shutdown()
	{
		try
		{
			socket.close();
		}
		catch( Exception e ) { };
	}

	// Receive a HTTP request
	public HTTPRequest getRequest( BufferedReader text ) throws IOException
	{
		String req = "", s;
		while( ( s = text.readLine() ) != null && !s.equals( "" ) )
		{
			req += s + '\n';
		}
		StringTokenizer st = new StringTokenizer( req, "\n", false );
		String[] request = new String[ st.countTokens() ];
		for( int i = 0; i < request.length; i++ )
		{
			request[ i ] = st.nextToken();
		}
		return new HTTPRequest( request );
	}

	// Response to a HTTP request
	public void sendResponse( PrintStream text, OutputStream bin ) throws IOException
	{
		File f = new File( (new File( wwwroot + "/" + getURLPath( request.resource ) )).getCanonicalPath() );
		if( !request.command.toUpperCase().equals( "GET" ) )
		{
			text.print( Config.CONNECTION_RESPONSE_NOT_IMPLEMENTED );
			return;
		}
		if( !f.canRead() || !f.getPath().startsWith( wwwroot ) )
		{
			text.print( Config.CONNECTION_RESPONSE_NOT_FOUND );
			return;
		}
		if( f.isDirectory() )
		{
			Config.CONNECTION_RESPONSE_DIRECTORY( /*request.host, */request.resource, text, f.listFiles() );
		}
		else sendFile( f, text, bin );
	}

	private void sendFile( File f, PrintStream text, OutputStream bin ) throws IOException
	{
		// determinate the content type
		int i = f.getName().lastIndexOf( '.' );
		String ext = ( ( i == -1 ) ? "" : f.getName().substring( i + 1 ) ).toLowerCase() + ' ';
		String content_type = "";
		for( i = 0; i < Config.CONNECTION_FILE_TYPE.length; i++ )
		{
			if( Config.CONNECTION_FILE_TYPE[ i ].indexOf( ext ) != -1 )
			{
				content_type = Config.CONNECTION_CONTENT_TYPE[ i ];
				break;
			}
		}
		if( content_type.equals( "" ) ) content_type = Config.CONNECTION_CONTENT_TYPE[ Config.CONNECTION_CONTENT_TYPE.length - 1 ];
		text.print( Config.CONNECTION_RESPONSE_HEADER + content_type + Config.CONNECTION_RESPONSE_FOOTER );

		// send file data
		FileInputStream file = new FileInputStream( f );
		byte[] fbuf = new byte[ Config.CONNECTION_FILEBUFFER_SIZE ];
		while( ( i = file.read( fbuf ) ) > 0 )
		{
			bin.write( fbuf, 0, i );
		}
		file.close();
	}

	private String getURLPath( String url )
	{
		int i = url.indexOf( "://" );
		if( i == -1 ) return url;
		return url.substring( url.indexOf( '/', i + 3 ) );
	}
}

class HTTPRequest
{
	public String command = "", resource = "", protocol = "";
	public String[] accept = new String[ 0 ];
	public String language = "";
	public String[] encoding = new String[ 0 ];
	public String user_agent = "";
	public String host = "";
	public String connection = "";
	public String cache_control = "";

	public HTTPRequest( String[] request )
	{
		if( request.length < 1 ) return;
		StringTokenizer st = new StringTokenizer( request[ 0 ], " ", false );
		try
		{
			command = st.nextToken();
			resource = URLDecoder.decode( st.nextToken() );
			protocol = st.nextToken();
		}
		catch( NoSuchElementException e ) { }
/*		for( int i = 1; i < request.length; i++ )
		{
			if( request[ i ].toUpperCase().startsWith( "HOST: " ) )
			{
				host = request[ i ].substring( request[ i ].indexOf( ' ' ) + 1 );
				break;
			}
		}*/
	}
}