import java.io.*;
import java.util.*;

/*
Realize a command line debugger/monitor, which accepts the following commands:
  ? - print available commands and description for each
  c - close connection(s)
  v - view current connections
  q - close all connections and terminate the server
*/
public class Debugger extends Thread
{
	private ThreadGroup connections;

	private static String readLine()
	{
		try
		{
			return (new BufferedReader( new InputStreamReader( System.in ) )).readLine();
		}
		catch( Exception e ) { }
		return "";
	}

	private void printCmdHelp()
	{
		System.out.print(
			"  c [string] close all connections which start with <string>\n" +
			"  v          view current connections\n" +
			"  q          shutdown the server\n" );
	}

	private void printConnections()
	{
		Thread[] list = new Thread[ connections.activeCount() ];
		int n = connections.enumerate( list );
		for( int i = 0; i < n; i++ )
		{
			System.out.println( list[ i ].toString() );
		}
		System.out.println( "  " + n + " connections" );
	}

	private void closeConnections( String con )
	{
		Thread[] list = new Thread[ connections.activeCount() ];
		int n = connections.enumerate( list );
		for( int i = 0; i < n; i++ )
		{
			String s = list[ i ].toString();
			if( !s.startsWith( con ) ) continue;
			System.out.println( "  Closing " + s );
			((Connection)list[ i ]).shutdown();
		}
	}

	private String[] parseCmd( String cmd )
	{
		StringTokenizer st = new StringTokenizer( cmd, " ", false );
		String[] s = new String[ st.countTokens() ];
		try
		{
			for( int i = 0; i < s.length; i++ ) s[ i ] = st.nextToken();
		}
		catch( NoSuchElementException e ) { }
		return s;
	}

	public Debugger( ThreadGroup connections )
	{
		this.connections = connections;
	}
	public void run()
	{
		String[] cmd;
		System.out.println( "--  WebServer debugger/monitor (type ? for available commands) --\n" );
		for( ; ; )
		{
			System.out.print( "# " );
			cmd = parseCmd( readLine().trim() );
			if( cmd.length == 0 || cmd[ 0 ].length() > 1 ) continue;

			switch( Character.toLowerCase( cmd[ 0 ].charAt( 0 ) ) )
			{
				case '?' : printCmdHelp(); break;
				case 'c' : closeConnections( (cmd.length > 1)? cmd[ 1 ] : "" ); break;
				case 'v' : printConnections(); break;
				case 'q' : closeConnections( "" ); System.exit( 0 );
			}
		}

	}

}