import java.io.*;
import java.net.*;

/*
The basic WebServer class:
  - parses command line parameters;
  - holds the main loop, which waits for incomming connections
*/
public class WebServer
{
	// Server socket for incomming connections
	private static ServerSocket socketListener;
	// Webservice root directory
	private static String serverwwwroot = Config.WEBSERVER_DEFAULT_ROOT;
	// ServerSocket listening port
	private static int serverPort = Config.WEBSERVER_DEFAULT_PORT;
	// ThreadGroup for all connections to the server
	private static ThreadGroup serverConnections;

	// Listen for a new connection, and create a thread to undertake.
	// The new thread joins to serverConnections thread group.
	private static void startServer()
	{
		for( ; ; )
		{
			try
			{
				(new Connection( serverConnections, socketListener.accept(), serverwwwroot )).start();
			}
			catch( Exception e )
			{
			}
		}
	}

	private static void printCmdLineHelp()
	{
		printCmdLineHelp( null );
	}
	private static void printCmdLineHelp( String errmsg )
	{
		System.out.println(
			( ( errmsg != null )? "\nCommand line error: " + errmsg : "" ) +
			"\nUsage: WebServer wwwroot [port]\n"
		);
		System.exit( 1 );
	}

	// Command line arguments: WebServer wwwroot [port]
	public static void main( String[] arg ) throws MalformedURLException
	{
		if( arg.length == 0 || arg.length > 2 ) printCmdLineHelp();
		try
		{
			serverwwwroot = (new File( arg[ 0 ] )).getCanonicalPath();
			if( !(new File( serverwwwroot )).isDirectory() ) printCmdLineHelp( "<wwwroot> must be directory" );
		}
		catch( IOException e )
		{
			printCmdLineHelp( "<wwwroot> must be directory" );
		}
		if( arg.length > 1 )
		{
			try
			{
				serverPort = Integer.parseInt( arg[ 1 ] );
			}
			catch( NumberFormatException e )
			{
				printCmdLineHelp( "<port> must be an integer number" );
			}
			if( serverPort < 0 || serverPort > 65535 )
			{
				printCmdLineHelp( "<port> must be an integer number between 0 and 65535" );
			}
		}

		try
		{
			socketListener = new ServerSocket( serverPort, Config.WEBSERVER_MAXCONNECTIONS );
		}
		catch( IOException e )
		{
			System.out.println( "An I/O error occured when opening the socket" );
			System.exit( 2 );
		}

		serverConnections = new ThreadGroup( "Server connections" );
		(new Debugger( serverConnections )).start();
		startServer();
	}
}